/*********************************************************************
*                SEGGER Microcontroller GmbH & Co. KG                *
*        Solutions for real time microcontroller applications        *
**********************************************************************
*                                                                    *
*        (c) 1996 - 2017  SEGGER Microcontroller GmbH & Co. KG       *
*                                                                    *
*        Internet: www.segger.com    Support:  support@segger.com    *
*                                                                    *
**********************************************************************

** emWin V5.42 - Graphical user interface for embedded applications **
All  Intellectual Property rights  in the Software belongs to  SEGGER.
emWin is protected by  international copyright laws.  Knowledge of the
source code may not be used to write a similar product.  This file may
only be used in accordance with the following terms:

The software has been licensed to  ARM LIMITED whose registered office
is situated at  110 Fulbourn Road,  Cambridge CB1 9NJ,  England solely
for  the  purposes  of  creating  libraries  for  ARM7, ARM9, Cortex-M
series,  and   Cortex-R4   processor-based  devices,  sublicensed  and
distributed as part of the  MDK-ARM  Professional  under the terms and
conditions  of  the   End  User  License  supplied  with  the  MDK-ARM
Professional. 
Full source code is available at: www.segger.com

We appreciate your understanding and fairness.
----------------------------------------------------------------------
Licensing information
Licensor:                 SEGGER Software GmbH
Licensed to:              ARM Ltd, 110 Fulbourn Road, CB1 9NJ Cambridge, UK
Licensed SEGGER software: emWin
License number:           GUI-00181
License model:            LES-SLA-20007, Agreement, effective since October 1st 2011 
Licensed product:         MDK-ARM Professional
Licensed platform:        ARM7/9, Cortex-M/R4
Licensed number of seats: -
----------------------------------------------------------------------
File        : LCD_X.c
Purpose     : Port routines for EFM32GG DK3750 RGB & SPI
----------------------------------------------------------------------
*/

#include "GUI.h"
#include "LCD_X.h"

/*********************************************************************
*
*       Hardware configuration
*
**********************************************************************
*/

#include "em_device.h"
#include "em_ebi.h"
#include "em_cmu.h"
#include "em_gpio.h"
#include "em_usart.h"
#include "bsp.h"

/* Configure TFT direct drive from EBI BANK2 */
static const EBI_TFTInit_TypeDef tftInit =
{ ebiTFTBank2,                  /* Select EBI Bank 2 */
  ebiTFTWidthHalfWord,          /* Select 2-byte (16-bit RGB565) increments */
  ebiTFTColorSrcMem,            /* Use memory as source for mask/blending */
  ebiTFTInterleaveUnlimited,    /* unlimited interleaved accesses allowed */
  ebiTFTFrameBufTriggerVSync,   /* VSYNC as frame buffer update trigger */
  false,                        /* Drive DCLK from negative edge of internal clock */
  ebiTFTMBDisabled,             /* No masking and alpha blending enabled */
  ebiTFTDDModeExternal,         /* Drive from external memory */
  ebiActiveLow,                 /* CS Active Low polarity */
  ebiActiveHigh,                /* DCLK Active High polarity */
  ebiActiveLow,                 /* DATAEN Active Low polarity */
  ebiActiveLow,                 /* HSYNC Active Low polarity */
  ebiActiveLow,                 /* VSYNC Active Low polarity */
  320,                          /* Horizontal size in pixels */
  20,                           /* Horizontal Front Porch */
  30,                           /* Horizontal Back Porch */
  2,                            /* Horizontal Synchronization Pulse Width */
  240,                          /* Vertical size in pixels */
  2,                            /* Vertical Front Porch */
  4,                            /* Vertical Back Porch */
  2,                            /* Vertical Synchronization Pulse Width */
  0x0000,                       /* Frame Address pointer offset to EBI memory base */
  5,                            /* DCLK Period */
  0,                            /* DCLK Start cycles */
  0,                            /* DCLK Setup cycles */
  0,                            /* DCLK Hold cycles */
};

/** Set default values for tft synchronous mode init structure */
static const USART_InitSync_TypeDef usartInit =
{ usartEnable,                  /* Enable RX/TX when init completed. */
  48000000,                     /* Use 48MHz reference clock */
  1000000,                      /* 7 Mbits/s. */
  usartDatabits9,               /* 9 databits. */
  true,                         /* Master mode. */
  true,                         /* Send most significant bit first. */
  usartClockMode3,              /* Clock idle low, sample on rising edge. */
  false,
  usartPrsRxCh0,
  false 
};

/*********************************************************************
*
*       Private code
*
*********************************************************************
*/

/* Initialize SPI interface to TFT-LCD SSD2119 controller */
static void SPI_Init(void) {

  /* Enabling clock to USART */
  CMU_ClockEnable(cmuClock_HFPER, true);
  CMU_ClockEnable(cmuClock_USART1, true);
  CMU_ClockEnable(cmuClock_GPIO, true);

  /* IO configuration (USART 1, Location #1) */
  GPIO_PinModeSet(gpioPortD, 0, gpioModePushPull, 0); /* TX - MOSI */
  GPIO_PinModeSet(gpioPortD, 1, gpioModeInput, 0);    /* RX - MISO */
  GPIO_PinModeSet(gpioPortD, 2, gpioModePushPull, 0); /* CLK */
  GPIO_PinModeSet(gpioPortD, 3, gpioModePushPull, 1); /* CS  */

  /* Ensure out of reset configuration */
  USART_Reset(USART1);

  /* Initialize USART1, in SPI master mode. */
  USART_InitSync(USART1, &usartInit);

  USART1->ROUTE =
    USART_ROUTE_TXPEN |
    USART_ROUTE_RXPEN |
    USART_ROUTE_CLKPEN |
    USART_ROUTE_LOCATION_LOC1;
}

static void EnableVsyncInterrupt(void) {

  /* Disable all EBI interrupts */
  EBI_IntDisable(_EBI_IF_MASK);

  /* Clear previous interrupts */
  EBI_IntClear(_EBI_IF_MASK);

  /* Enable VSYNC interrupt */
  EBI_IntEnable(EBI_IF_VFPORCH);

  /* Enable EBI interrupts in core */
  NVIC_ClearPendingIRQ(EBI_IRQn);
  NVIC_EnableIRQ(EBI_IRQn);
}

/*********************************************************************
*
*       Exported code
*
*********************************************************************
*/

/* VSYNC interrupt handler. Will send confirmation to emWin that frame buffer
 * addres is changed. Frame buffer address is automatically reloaded during VSYNC.
 */
void EBI_IRQHandler(void) {
  uint32_t flags;

  flags = EBI_IntGet();
  EBI_IntClear(flags);

  if (flags & EBI_IF_VFPORCH) {
    if (LCD_X_PendingBuffer >= 0) {

      /* Send a confirmation that the buffer is visible now */
      GUI_MULTIBUF_Confirm(LCD_X_PendingBuffer);

      LCD_X_PendingBuffer = -1;
    }
  }
}

/*********************************************************************
*
*       LCD_X_Init
*
* Purpose:
*   This routine should be called from your application program
*   to set port pins to their initial values
*/
void LCD_X_Init(void) {
  uint32_t i, freq;

  /* Initialize EBI banks (Board Controller, external PSRAM, ..) */
  BSP_Init(BSP_INIT_DK_EBI);

  /* Configure for 48MHz HFXO operation of core clock */
  CMU_ClockSelectSet(cmuClock_HF, cmuSelect_HFXO);

  CMU_ClockEnable(cmuClock_GPIO, true);
  CMU_ClockEnable(cmuClock_ADC0, true);

  /* Enable SSD2119 Serial Port Interface */
  BSP_PeripheralAccess(BSP_TFT, true);

  /* Enable EBI mode of operation on SSD2119 controller */
  BSP_DisplayControl(BSP_Display_EBI);
  BSP_DisplayControl(BSP_Display_ResetAssert);
  BSP_DisplayControl(BSP_Display_PowerDisable);
  freq = SystemCoreClockGet();
  for (i = 0; i < (freq / 100); i++) {
    __NOP();
  }
  /* Configure display for Direct Drive "Mode Generic" + 3-wire SPI mode */
  BSP_DisplayControl(BSP_Display_ModeGeneric);
  BSP_DisplayControl(BSP_Display_PowerEnable);
  BSP_DisplayControl(BSP_Display_ResetRelease);

  SPI_Init();

  /* Configure GPIO for EBI and TFT */
  /* EBI TFT DCLK/Dot Clock */
  GPIO_PinModeSet(gpioPortA, 8,  gpioModePushPull, 0);
  /* EBI TFT DATAEN */
  GPIO_PinModeSet(gpioPortA, 9,  gpioModePushPull, 0);
  /* EBI TFT VSYNC  */
  GPIO_PinModeSet(gpioPortA, 10, gpioModePushPull, 0);
  /* EBI TFT HSYNC */
  GPIO_PinModeSet(gpioPortA, 11, gpioModePushPull, 0);

  /* Configure EBI TFT direct drive */
  EBI_TFTInit(&tftInit);
  EnableVsyncInterrupt();
}

/*********************************************************************
*
*       LCD_X_SetVRAMAddr
*
* Purpose:
*   Should set the frame buffer base address
*/
void LCD_X_SetVRAMAddr(U32 addr) {
  EBI_TFTFrameBaseSet(addr & 0x03FFFFFF);
}

/*********************************************************************
*
*       LCD_X_ClrCS
*
* Purpose:
*   Sets the chip select pin to low
*/
void LCD_X_ClrCS(void) {
  GPIO_PinOutClear(gpioPortD, 3);
}

/*********************************************************************
*
*       LCD_X_SetCS
*
* Purpose:
*   Sets the chip select pin to high
*/
void LCD_X_SetCS(void) {
  GPIO_PinOutSet(gpioPortD, 3);
}

/*********************************************************************
*
*       LCD_X_WriteM
*
* Purpose:
*   Writes multiple bytes to controller
*/
void LCD_X_WriteM(U8 * pData, int NumBytes) {

  if (NumBytes == 0) return;

  /* Select register */
  USART1->CTRL = USART1->CTRL & ~USART_CTRL_BIT8DV;
  USART_Tx(USART1, *pData++);
  USART_Rx(USART1);

  /* Write data */
  USART1->CTRL = USART1->CTRL |  USART_CTRL_BIT8DV;
  while (--NumBytes) {
    USART_Tx(USART1, *pData++);
    USART_Rx(USART1);
  }
}

/*************************** End of file ****************************/
