/* -----------------------------------------------------------------------------
 * Copyright (c) 2013-2015 ARM Ltd.
 *
 * This software is provided 'as-is', without any express or implied warranty.
 * In no event will the authors be held liable for any damages arising from
 * the use of this software. Permission is granted to anyone to use this
 * software for any purpose, including commercial applications, and to alter
 * it and redistribute it freely, subject to the following restrictions:
 *
 * 1. The origin of this software must not be misrepresented; you must not
 *    claim that you wrote the original software. If you use this software in
 *    a product, an acknowledgment in the product documentation would be
 *    appreciated but is not required.
 *
 * 2. Altered source versions must be plainly marked as such, and must not be
 *    misrepresented as being the original software.
 *
 * 3. This notice may not be removed or altered from any source distribution.
 *
 *
 * $Date:        22. June 2015
 * $Revision:    V2.1
 *
 * Driver:       Driver_SPI0, Driver_SPI1
 * Configured:   via RTE_Device.h configuration file
 * Project:      SPI (USART used for SPI) Driver for SiliconLabs EFM32GG
 * --------------------------------------------------------------------------
 * Use the following configuration settings in the middleware component
 * to connect to this driver.
 *
 *   Configuration Setting                 Value   SPI Interface
 *   ---------------------                 -----   -------------
 *   Connect to hardware via Driver_SPI# = 0       use SPI0 (USART0)
 *   Connect to hardware via Driver_SPI# = 1       use SPI1 (USART1)
 * -------------------------------------------------------------------------- */

/* History:
 *  Version 2.1
 *    - PowerControl for Power OFF and Uninitialize functions made unconditional.
 *    - Corrected status bit-field handling, to prevent race conditions.
 *  Version 2.0
 *    - Initial CMSIS Driver API V2.00 release
 */


/* NOTE:
 *   Nine data bit transfer is not supported.
 */

#include <string.h>

#include "em_device.h"

#include "em_cmu.h"
#include "em_dma.h"
#include "em_gpio.h"
#include "em_usart.h"

#include "Driver_SPI.h"

#include "RTE_Device.h"
#include "RTE_Components.h"

#define ARM_SPI_DRV_VERSION ARM_DRIVER_VERSION_MAJOR_MINOR(2,1)   // driver version

// Current driver status flag definition
#define SPI_INITIALIZED           (1U << 0)     // SPI initialized
#define SPI_POWERED               (1U << 1)     // SPI powered on
#define SPI_CONFIGURED            (1U << 2)     // SPI configured

#define DMA_CH_CTRL_SIGSEL_USARTRXDATAV (0x00)
#define DMA_CH_CTRL_SIGSEL_USARTTXBL    (0x01)

// Driver Version
static const ARM_DRIVER_VERSION DriverVersion = {
  ARM_SPI_API_VERSION,
  ARM_SPI_DRV_VERSION
};

// Driver Capabilities
static const ARM_SPI_CAPABILITIES DriverCapabilities = {
  1,  // Simplex Mode (Master and Slave)
  0,  // TI Synchronous Serial Interface
  0,  // Microwire Interface
  1   // Signal Mode Fault event: \ref ARM_SPI_EVENT_MODE_FAULT
};

// SPI STAUS
typedef struct _SPI_STATUS {
  uint8_t busy;                         // Transmitter/Receiver busy flag
  uint8_t data_lost;                    // Data lost: Receive overflow / Transmit underflow (cleared on start of transfer operation)
  uint8_t mode_fault;                   // Mode fault detected; optional (cleared on start of transfer operation)
} SPI_STATUS;

// SPI Information (Run-time)
typedef struct _SPI_INFO {
  ARM_SPI_SignalEvent_t cb_event;       // Event Callback
  SPI_STATUS            status;         // Status flags
  uint8_t               state;          // Current SPI state
  uint32_t              mode;           // Current SPI mode
  uint8_t               cfg;            // SPI configuration flags
} SPI_INFO;

// SPI Transfer Information (Run-Time)
typedef struct _SPI_TRANSFER_INFO {
  uint32_t              num;            // Total number of transfers
  uint8_t              *rx_buf;         // Pointer to in data buffer
  uint8_t              *tx_buf;         // Pointer to out data buffer
  uint32_t              rx_cnt;         // Number of data received
  uint32_t              tx_cnt;         // Number of data sent
  uint32_t              dump_val;       // Variable for dumping DMA data
  uint16_t              def_val;        // Default transfer value
} SPI_TRANSFER_INFO;

// GPIO pin description
typedef struct _GPIO_PIN_ID {
  GPIO_Port_TypeDef     port;
  uint8_t               num;
} GPIO_PIN_ID;


// SPI Resources definition 
typedef struct {
        USART_TypeDef     *reg;                 // Pointer to USART peripheral
  enum  IRQn               usart_rx_irq;        // USART Rx IRQ Number
  enum  IRQn               usart_tx_irq;        // USART Tx IRQ Number
        GPIO_PIN_ID        pin_rx;              // RX/MISO Pin identifier
        GPIO_PIN_ID        pin_tx;              // TX/MOSI Pin identifier
        GPIO_PIN_ID        pin_clk;             // CLK Pin identifier
        GPIO_PIN_ID        pin_cs;              // CS Pin identifier
        uint8_t            pin_cs_en;           // CS pin enabled
        uint8_t            pin_loc;             // USART Pin Location
        uint8_t            dma_rx_ch;           // USART Receive DMA channel
        uint8_t            dma_tx_ch;           // USART Transmit DMA channel
        uint32_t           dma_sourcesel;       // DMA channel input source
        uint8_t            dma_en;              // DMA enabled
        uint32_t           clock_limit;         // Maximum SPI bus frequency
        CMU_Clock_TypeDef  clock_mask;          // Clock register mask
        SPI_INFO          *info;                // Run-Time information
        SPI_TRANSFER_INFO *xfer;                // SPI transfer information
} const SPI_RESOURCES;

// DMA Callback function
extern DMA_DESCRIPTOR_TypeDef DMA_DescArray[];

// DMA Callback function
void USART_DMA_Complete (uint32_t channel, bool primary, void *ptr_spi);

// DMA Callback
DMA_CB_TypeDef USART_DMA_Cb = {
  USART_DMA_Complete,
  NULL,
  0
};

// SPI0
#if (RTE_USART0 && RTE_USART0_SPI_MODE)

#if    (RTE_USART0_CLK_PIN == 0)
#error "CLK Pin is disabled"
#endif

#ifndef __USART0_CLK_MAX_FREQUENCY
  // This definition is used to limit maximum bus clock frequency
  #define __USART0_CLK_MAX_FREQUENCY    0
#endif

// SPI0 Information
static SPI_INFO           SPI0_Info;
static SPI_TRANSFER_INFO  SPI0_TransferInfo;

static SPI_RESOURCES SPI0_Resources = {
  USART0,
  USART0_RX_IRQn,
  USART0_TX_IRQn,
  { RTE_USART0_RX_PORT,  RTE_USART0_RX_BIT },
  { RTE_USART0_TX_PORT,  RTE_USART0_TX_BIT },
  { RTE_USART0_CLK_PORT, RTE_USART0_CLK_BIT},
  { RTE_USART0_CS_PORT,  RTE_USART0_CS_BIT },
  RTE_USART0_CS_PIN,
  RTE_USART0_PIN_LOC,
  RTE_USART0_RX_DMA_CHANNEL,
  RTE_USART0_TX_DMA_CHANNEL,
  DMA_CH_CTRL_SOURCESEL_USART0,
#if (RTE_USART0_DMA)
  1,
#else
  0,
#endif
  __USART0_CLK_MAX_FREQUENCY,
  cmuClock_USART0,
  &SPI0_Info,
  &SPI0_TransferInfo
};
#endif

// SPI1
#if (RTE_USART1 && RTE_USART1_SPI_MODE)

#if    (RTE_USART1_CLK_PIN == 0)
#error "CLK Pin is disabled"
#endif

#ifndef __USART1_CLK_MAX_FREQUENCY
  // This definition is used to limit maximum bus clock frequency
  #define __USART1_CLK_MAX_FREQUENCY    0
#endif

// SPI1 Information
static SPI_INFO           SPI1_Info;
static SPI_TRANSFER_INFO  SPI1_TransferInfo;

static SPI_RESOURCES SPI1_Resources = {
  USART1,
  USART1_RX_IRQn,
  USART1_TX_IRQn,
  { RTE_USART1_RX_PORT,  RTE_USART1_RX_BIT },
  { RTE_USART1_TX_PORT,  RTE_USART1_TX_BIT },
  { RTE_USART1_CLK_PORT, RTE_USART1_CLK_BIT},
  { RTE_USART1_CS_PORT,  RTE_USART1_CS_BIT },
  RTE_USART1_CS_PIN,
  RTE_USART1_PIN_LOC,
  RTE_USART1_RX_DMA_CHANNEL,
  RTE_USART1_TX_DMA_CHANNEL,
  DMA_CH_CTRL_SOURCESEL_USART1,
#if (RTE_USART1_DMA)
  1,
#else
  0,
#endif
  __USART1_CLK_MAX_FREQUENCY,
  cmuClock_USART1,
  &SPI1_Info,
  &SPI1_TransferInfo
};
#endif

// SPI2
#if (RTE_USART2 && RTE_USART2_SPI_MODE)

#if    (RTE_USART2_CLK_PIN == 0)
#error "CLK Pin is disabled"
#endif

#ifndef __USART2_CLK_MAX_FREQUENCY
  // This definition is used to limit maximum bus clock frequency
  #define __USART2_CLK_MAX_FREQUENCY    0
#endif

// SPI2 Information
static SPI_INFO           SPI2_Info;
static SPI_TRANSFER_INFO  SPI2_TransferInfo;

static SPI_RESOURCES SPI2_Resources = {
  USART2,
  USART2_RX_IRQn,
  USART2_TX_IRQn,
  { RTE_USART2_RX_PORT,  RTE_USART2_RX_BIT },
  { RTE_USART2_TX_PORT,  RTE_USART2_TX_BIT },
  { RTE_USART2_CLK_PORT, RTE_USART2_CLK_BIT},
  { RTE_USART2_CS_PORT,  RTE_USART2_CS_BIT },
  RTE_USART2_CS_PIN,
  RTE_USART2_PIN_LOC,
  RTE_USART2_RX_DMA_CHANNEL,
  RTE_USART2_TX_DMA_CHANNEL,
  DMA_CH_CTRL_SOURCESEL_USART2,
#if (RTE_USART2_DMA)
  1,
#else
  0,
#endif
  __USART2_CLK_MAX_FREQUENCY,
  cmuClock_USART2,
  &SPI2_Info,
  &SPI2_TransferInfo
};
#endif

// Local functions
/**
  \fn          void SPI_DMA_Transfer (SPI_RESOURCES *spi)
  \brief       Set new DMA transfer.
  \param[in]   spi     Pointer to SPI resources
  \return      \ref ARM_DRV_VERSION
*/
static void SPI_DMA_Transfer (SPI_RESOURCES *spi) {
  DMA_CfgDescr_TypeDef DmaDescCfg;
  uint32_t num;
  void *data, *data_reg;

  // Number of data to transfer
  num = spi->xfer->num - spi->xfer->tx_cnt;

  // Limit DMA transfer to 1024
  if (num > 1024) num = 1024;

  // RX DMA
  if ((spi->reg->FRAME & _USART_FRAME_DATABITS_MASK) > USART_FRAME_DATABITS_EIGHT) {
    DmaDescCfg.dstInc  = dmaDataInc2;
    DmaDescCfg.size    = dmaDataSize2;
  }
  else {
    DmaDescCfg.dstInc  = dmaDataInc1;
    DmaDescCfg.size    = dmaDataSize1;
  }

  if ((spi->reg->FRAME & _USART_FRAME_DATABITS_MASK) > USART_FRAME_DATABITS_EIGHT) {
    data_reg           = (void *)&spi->reg->RXDOUBLE;
  }
  else {
    data_reg           = (void *)&spi->reg->RXDATA;
  }

  if (spi->xfer->rx_buf == NULL) {
    DmaDescCfg.dstInc  = dmaDataIncNone;
    data               = (void *)&spi->xfer->dump_val;
  } else {
    data               = (void *)spi->xfer->rx_buf;
  }

  DmaDescCfg.srcInc  = dmaDataIncNone;
  DmaDescCfg.arbRate = dmaArbitrate1;
  DmaDescCfg.hprot   = 0;

  DMA_CfgDescr (spi->dma_rx_ch, true, &DmaDescCfg);

  DMA_ActivateBasic (spi->dma_rx_ch, true, false,
                     data, data_reg, num-1);

  // TX DMA
  if ((spi->reg->FRAME & _USART_FRAME_DATABITS_MASK) > USART_FRAME_DATABITS_EIGHT) {
    DmaDescCfg.srcInc  = dmaDataInc2;
    DmaDescCfg.size    = dmaDataSize2;
  }
  else {
    DmaDescCfg.srcInc  = dmaDataInc1;
    DmaDescCfg.size    = dmaDataSize1;
  }

  if ((spi->reg->FRAME & _USART_FRAME_DATABITS_MASK) > USART_FRAME_DATABITS_EIGHT) {
    data_reg           = (void *)&spi->reg->TXDOUBLE;
  }
  else {
    data_reg           = (void *)&spi->reg->TXDATA;
  }

  if (spi->xfer->tx_buf == NULL) {
    DmaDescCfg.srcInc  = dmaDataIncNone;
    data               = (void *)&spi->xfer->def_val;
  } else {
    data               = (void *)spi->xfer->tx_buf;
  }

  DmaDescCfg.dstInc  = dmaDataIncNone;
  DmaDescCfg.arbRate = dmaArbitrate1;
  DmaDescCfg.hprot   = 0;

  DMA_CfgDescr (spi->dma_tx_ch, true, &DmaDescCfg);

  DMA_ActivateBasic (spi->dma_tx_ch, true, false,
                     data_reg, data, num-1);
}

// Driver Functions

/**
  \fn          ARM_DRIVER_VERSION SPI_GetVersion (void)
  \brief       Get SPI driver version.
  \return      \ref ARM_DRV_VERSION
*/
static ARM_DRIVER_VERSION SPI_GetVersion (void) {
  return DriverVersion;
}


/**
  \fn          ARM_SPI_CAPABILITIES SPI_GetCapabilities (void)
  \brief       Get driver capabilities.
  \return      \ref ARM_SPI_CAPABILITIES
*/
static ARM_SPI_CAPABILITIES SPI_GetCapabilities (void) {
  return DriverCapabilities;
}


/**
  \fn          int32_t SPI_Initialize (ARM_SPI_SignalEvent_t cb_event,
                                       SPI_RESOURCES        *spi)
  \brief       Initialize SPI Interface.
  \param[in]   cb_event  Pointer to \ref ARM_SPI_SignalEvent
  \param[in]   spi       Pointer to SPI resources
  \return      \ref execution_status
*/
static int32_t SPI_Initialize (ARM_SPI_SignalEvent_t  cb_event,
                               SPI_RESOURCES         *spi) {

  DMA_Init_TypeDef DmaInit;

  if (spi->info->state & SPI_INITIALIZED) return ARM_DRIVER_OK;

  // Initialize SPI Run-Time Resources
  spi->info->cb_event          = cb_event;
  spi->info->status.busy       = 0U;
  spi->info->status.data_lost  = 0U;
  spi->info->status.mode_fault = 0U;

  // Clear transfer information
  memset(spi->xfer, 0, sizeof(SPI_TRANSFER_INFO));

  // Enable GPIO peripheral clock
  CMU_ClockEnable (cmuClock_GPIO, true);

  // Configure pins
  // Configure USART CLK pin (SCLK)
  GPIO_PinModeSet (spi->pin_clk.port, spi->pin_clk.num, gpioModePushPull, 0);

  // Configure USART RX pin (MISO)
  GPIO_PinModeSet (spi->pin_rx.port, spi->pin_rx.num, gpioModePushPull, 0);

  // Configure USART TX pin (MOSI)
  GPIO_PinModeSet (spi->pin_tx.port, spi->pin_tx.num, gpioModePushPull, 0);

  // DMA Initialization
  if (spi->dma_en) {
    DmaInit.hprot        = false;
    DmaInit.controlBlock = DMA_DescArray;
    DMA_Init (&DmaInit);
  }

  spi->info->state = SPI_INITIALIZED;

  return ARM_DRIVER_OK;
}

/**
  \fn          int32_t SPI_Uninitialize (SPI_RESOURCES *spi)
  \brief       De-initialize SPI Interface.
  \param[in]   spi  Pointer to SPI resources
  \return      \ref execution_status
*/
static int32_t SPI_Uninitialize (SPI_RESOURCES *spi) {
  // Disable pins
  spi->reg->ROUTE = 0;

  // Unconfigure USART Tx pin
  GPIO_PinModeSet (spi->pin_tx.port,  spi->pin_tx.num,  gpioModeDisabled, 0);
  // Unconfigure USART Rx pin
  GPIO_PinModeSet (spi->pin_rx.port,  spi->pin_rx.num,  gpioModeDisabled, 0);
  // Unconfigure USART CLK pin
  GPIO_PinModeSet (spi->pin_clk.port, spi->pin_clk.num, gpioModeDisabled, 0);

  if (spi->pin_cs_en) {
    // Unconfigure USART CS pin
    GPIO_PinModeSet (spi->pin_cs.port,  spi->pin_cs.num,  gpioModeDisabled, 0);
  }

  spi->info->state = 0U;

  return ARM_DRIVER_OK;
}

/**
  \fn          int32_t SPI_PowerControl (ARM_POWER_STATE state,
                                         SPI_RESOURCES  *spi)
  \brief       Controls SPI Interface Power.
  \param[in]   state  Power state
  \param[in]   spi    Pointer to SPI resources
  \return      \ref execution_status
*/
static int32_t SPI_PowerControl (ARM_POWER_STATE  state,
                                 SPI_RESOURCES   *spi) {
  DMA_CfgChannel_TypeDef DmaCfg;

  switch (state) {
    case ARM_POWER_OFF:
      // Disable USART IRQ
      NVIC_DisableIRQ (spi->usart_tx_irq);
      NVIC_DisableIRQ (spi->usart_rx_irq);

      spi->info->status.busy       = 0U;
      spi->info->status.data_lost  = 0U;
      spi->info->status.mode_fault = 0U;

      // SPI not powered
      spi->info->state = SPI_INITIALIZED;

      // Disable USART clock
      CMU_ClockEnable (spi->clock_mask, false);
      break;

    case ARM_POWER_FULL:
      if (spi->info->state & SPI_POWERED) { return ARM_DRIVER_OK; }

      // Enable USARTx peripheral clock
      CMU_ClockEnable (spi->clock_mask, true);

      // Define USARTx pin location and enable Tx, Rx and CLK pins
      spi->reg->ROUTE = (spi->reg->ROUTE & ~_USART_ROUTE_LOCATION_MASK)    |
                        (spi->pin_loc      << _USART_ROUTE_LOCATION_SHIFT) |
                         USART_ROUTE_RXPEN | USART_ROUTE_TXPEN | USART_ROUTE_CLKPEN;

      // Disable receiver and transmitter.
      spi->reg->CMD = USART_CMD_TXDIS  |
                      USART_CMD_RXDIS  |
                      USART_CMD_TXTRIEN;

      // Enable synchronous mode
      spi->reg->CTRL = USART_CTRL_SYNC;

      // Disable USART interrupts and clear pending interrupt flags
      spi->reg->IEN = 0;
      spi->reg->IFC = _USART_IFC_MASK;

      // Initialize DMA channels
      if (spi->dma_en) {
        // Setup DMA channel configuration structure
        DmaCfg.cb        = &USART_DMA_Cb;
        DmaCfg.enableInt = true;
        DmaCfg.highPri   = false;

        DmaCfg.select = spi->dma_sourcesel | DMA_CH_CTRL_SIGSEL_USARTRXDATAV;
        // Configure channel
        DMA_CfgChannel (spi->dma_rx_ch, &DmaCfg);

        DmaCfg.select = spi->dma_sourcesel | DMA_CH_CTRL_SIGSEL_USARTTXBL;

        // Configure channel
        DMA_CfgChannel (spi->dma_tx_ch, &DmaCfg);
      }

      spi->info->status.busy       = 0U;
      spi->info->status.data_lost  = 0U;
      spi->info->status.mode_fault = 0U;

      // Clear transfer information
      memset(spi->xfer, 0, sizeof(SPI_TRANSFER_INFO));

      spi->info->state |= SPI_POWERED;

      // Enable USART interrupts
      NVIC_ClearPendingIRQ (spi->usart_tx_irq);
      NVIC_ClearPendingIRQ (spi->usart_rx_irq);
      NVIC_EnableIRQ (spi->usart_tx_irq);
      NVIC_EnableIRQ (spi->usart_rx_irq);
      break;
    default: return ARM_DRIVER_ERROR_UNSUPPORTED;
  }
  return ARM_DRIVER_OK;
}

/**
  \fn          int32_t SPI_Send (const void *data, uint32_t num, SPI_RESOURCES *spi)
  \brief       Start sending data to SPI transmitter.
  \param[in]   data  Pointer to buffer with data to send to SPI transmitter
  \param[in]   num   Number of data items to send
  \param[in]   spi  Pointer to SPI resources
  \return      \ref execution_status
*/
static int32_t SPI_Send (const void *data, uint32_t num, SPI_RESOURCES *spi) {

  if ((data == NULL) || (num == 0U))        { return ARM_DRIVER_ERROR_PARAMETER; }
  if (!(spi->info->state & SPI_CONFIGURED)) { return ARM_DRIVER_ERROR;           }
  if (  spi->info->status.busy)             { return ARM_DRIVER_ERROR_BUSY;      }

  spi->info->status.busy       = 1U;
  spi->info->status.data_lost  = 0U;
  spi->info->status.mode_fault = 0U;

  spi->xfer->rx_buf = NULL;
  spi->xfer->tx_buf = (uint8_t *)data;

  spi->xfer->num    = num;
  spi->xfer->rx_cnt = 0U;
  spi->xfer->tx_cnt = 0U;

  // Master simplex mode
  if ((spi->info->mode & ARM_SPI_CONTROL_Msk) == ARM_SPI_MODE_MASTER_SIMPLEX) {
    // Transmiter tristate disable
    spi->reg->CMD = USART_CMD_TXTRIDIS;
  }

  // DMA mode
  if (spi->dma_en) {
    SPI_DMA_Transfer (spi);
  } else {
  // Interrupt mode
    spi->reg->IEN |= (USART_IF_RXDATAV | USART_IF_TXBL);
  }

  return ARM_DRIVER_OK;
}

/**
  \fn          int32_t SPI_Receive (void *data, uint32_t num, SPI_RESOURCES *spi)
  \brief       Start receiving data from SPI receiver.
  \param[out]  data  Pointer to buffer for data to receive from SPI receiver
  \param[in]   num   Number of data items to receive
  \param[in]   spi   Pointer to SPI resources
  \return      \ref execution_status
*/
static int32_t SPI_Receive (void *data, uint32_t num, SPI_RESOURCES *spi) {

  if ((data == NULL) || (num == 0U))        { return ARM_DRIVER_ERROR_PARAMETER; }
  if (!(spi->info->state & SPI_CONFIGURED)) { return ARM_DRIVER_ERROR;           }
  if (  spi->info->status.busy)             { return ARM_DRIVER_ERROR_BUSY;      }

  spi->info->status.busy       = 1U;
  spi->info->status.data_lost  = 0U;
  spi->info->status.mode_fault = 0U;

  spi->xfer->rx_buf = (uint8_t *)data;
  spi->xfer->tx_buf = NULL;

  spi->xfer->num    = num;
  spi->xfer->rx_cnt = 0U;
  spi->xfer->tx_cnt = 0U;

  // Master simplex mode
  if ((spi->info->mode & ARM_SPI_CONTROL_Msk) == ARM_SPI_MODE_MASTER_SIMPLEX) {
    // Transmiter tristate enable
    spi->reg->CMD = USART_CMD_TXTRIEN;
  }

  // DMA mode
  if (spi->dma_en) {
    SPI_DMA_Transfer (spi);
  } else {
  // Interrupt mode
    spi->reg->IEN |= (USART_IF_RXDATAV | USART_IF_TXBL);
  }

  return ARM_DRIVER_OK;
}

/**
  \fn          int32_t SPI_Transfer (const void          *data_out,
                                           void          *data_in,
                                           uint32_t       num,
                                           SPI_RESOURCES *spi)
  \brief       Start sending/receiving data to/from SPI transmitter/receiver.
  \param[in]   data_out  Pointer to buffer with data to send to SPI transmitter
  \param[out]  data_in   Pointer to buffer for data to receive from SPI receiver
  \param[in]   num       Number of data items to transfer
  \param[in]   spi       Pointer to SPI resources
  \return      \ref execution_status
*/
static int32_t SPI_Transfer (const void *data_out, void *data_in, uint32_t num, SPI_RESOURCES *spi) {

  if ((data_out == NULL) || (data_in == NULL) || (num == 0U)) { return ARM_DRIVER_ERROR_PARAMETER; }
  if (!(spi->info->state & SPI_CONFIGURED))                   { return ARM_DRIVER_ERROR;           }
  if (  spi->info->status.busy)                               { return ARM_DRIVER_ERROR_BUSY;      }

  // If simplex mode,return error
  if ((spi->info->mode & ARM_SPI_CONTROL_Msk) == ARM_SPI_MODE_MASTER_SIMPLEX ||
      (spi->info->mode & ARM_SPI_CONTROL_Msk) == ARM_SPI_MODE_SLAVE_SIMPLEX) {
    return (ARM_DRIVER_ERROR);
  }

  spi->info->status.busy       = 1U;
  spi->info->status.data_lost  = 0U;
  spi->info->status.mode_fault = 0U;

  spi->xfer->rx_buf = (uint8_t *)data_in;
  spi->xfer->tx_buf = (uint8_t *)data_out;

  spi->xfer->num    = num;
  spi->xfer->rx_cnt = 0U;
  spi->xfer->tx_cnt = 0U;

  // DMA mode
  if (spi->dma_en) {
    SPI_DMA_Transfer (spi);
  } else {
  // Interrupt mode
    spi->reg->IEN |= (USART_IF_RXDATAV | USART_IF_TXBL);
  }

  return ARM_DRIVER_OK;
}

/**
  \fn          uint32_t SPI_GetDataCount (SPI_RESOURCES *spi)
  \brief       Get transferred data count.
  \param[in]   spi  Pointer to SPI resources
  \return      number of data items transferred
*/
static uint32_t SPI_GetDataCount (SPI_RESOURCES *spi) {

  if (!(spi->info->state & SPI_CONFIGURED)) { return 0U; }

  if (spi->xfer->rx_buf == NULL) {      // If send operation
    return spi->xfer->tx_cnt;
  } else {                              // If receive or Transfer operation
    return spi->xfer->rx_cnt;
  }
}

/**
  \fn          int32_t SPI_Control (uint32_t control, uint32_t arg, SPI_RESOURCES *spi)
  \brief       Control SPI Interface.
  \param[in]   control  Operation
  \param[in]   arg      Argument of operation (optional)
  \param[in]   spi      Pointer to SPI resources
  \return      common \ref execution_status and driver specific \ref spi_execution_status
*/
static int32_t SPI_Control (uint32_t control, uint32_t arg, SPI_RESOURCES *spi) {
  uint32_t mode, val;
  uint32_t ctrl, frame, cmd, ien;

  mode  = 0U;
  cmd   = 0U;
  ien   = 0U;
  frame = 0U;
  ctrl  = USART_CTRL_SYNC;

  if (!(spi->info->state & SPI_POWERED)) { return ARM_DRIVER_ERROR; }

  if ((control & ARM_SPI_CONTROL_Msk) == ARM_SPI_ABORT_TRANSFER) {
    // Disable interrupts
    spi->reg->IEN = 0;

    // Disable DMA channels
    if (spi->dma_en) {
      DMA_ChannelEnable (spi->dma_tx_ch, false);
      DMA_ChannelEnable (spi->dma_rx_ch, false);
    }    

    // Transmitter and receiver disable
    spi->reg->CMD = USART_CMD_TXDIS | USART_CMD_RXDIS |
                    USART_CMD_TXTRIEN;

    // Clear transmit and receive buffer and shift register
    spi->reg->CMD = USART_CMD_CLEARTX | USART_CMD_CLEARRX;

    memset(spi->xfer, 0, sizeof(SPI_TRANSFER_INFO));
    spi->info->status.busy = 0U;
    return ARM_DRIVER_OK;
  }

  if (  spi->info->status.busy) { return ARM_DRIVER_ERROR_BUSY; }

  switch (control & ARM_SPI_CONTROL_Msk) {
    case ARM_SPI_MODE_INACTIVE:
      mode |= ARM_SPI_MODE_INACTIVE;

      // Disable transmitter and receiver
      cmd |= USART_CMD_TXDIS | USART_CMD_RXDIS;
      break;

    case ARM_SPI_MODE_MASTER:
      mode |= ARM_SPI_MODE_MASTER;

      // Enable master mode, enable transmitter and receiver
      cmd |= USART_CMD_MASTEREN | USART_CMD_TXEN     |
             USART_CMD_RXEN     | USART_CMD_TXTRIDIS;
      break;

    case ARM_SPI_MODE_SLAVE:
      mode |= ARM_SPI_MODE_SLAVE;

      // Disable master mode, enable transmitter and receiver
      cmd |= USART_CMD_TXEN | USART_CMD_RXEN | USART_CMD_TXTRIDIS;

      // Enable TX underflow and RX overflow interrupts
      ien |= USART_IEN_TXUF | USART_IEN_RXOF;
      break;

    case ARM_SPI_MODE_MASTER_SIMPLEX:
      mode |= ARM_SPI_MODE_MASTER_SIMPLEX;

      // Loopbackmode
      ctrl |= USART_CTRL_LOOPBK;

      // Enable master mode, enable transmitter and receiver
      cmd = USART_CMD_MASTEREN | USART_CMD_TXEN     |
            USART_CMD_RXEN     | USART_CMD_TXTRIDIS;
      break;

    case ARM_SPI_MODE_SLAVE_SIMPLEX:
      mode |= ARM_SPI_MODE_SLAVE_SIMPLEX;

      // Loopbackmode
      ctrl |= USART_CTRL_LOOPBK;

      // Enable enable transmitter and receiver
      cmd = USART_CMD_TXEN | USART_CMD_RXEN | USART_CMD_TXTRIDIS;

      // Enable TX underflow and RX overflow interrupts
      ien |= USART_IEN_TXUF | USART_IEN_RXOF;
      break;

    case ARM_SPI_SET_BUS_SPEED:
      if (spi->clock_limit && (arg > spi->clock_limit)) {
        arg = spi->clock_limit;
      }
      USART_BaudrateSyncSet (spi->reg, 0, arg);
      break;

    case ARM_SPI_GET_BUS_SPEED:
      return (USART_BaudrateGet (spi->reg));

    case ARM_SPI_SET_DEFAULT_TX_VALUE:
      spi->xfer->def_val = (uint16_t)(arg & 0xFFFF);
      return ARM_DRIVER_OK;

    case ARM_SPI_CONTROL_SS:
      if (((spi->info->mode & ARM_SPI_CONTROL_Msk)        != ARM_SPI_MODE_MASTER)  ||
          ((spi->info->mode & ARM_SPI_SS_MASTER_MODE_Msk) != ARM_SPI_SS_MASTER_SW)) {
        return ARM_DRIVER_ERROR;
      }
      if (arg == ARM_SPI_SS_INACTIVE)
        GPIO_PinOutSet   (spi->pin_cs.port, spi->pin_cs.num);
      else
        GPIO_PinOutClear (spi->pin_cs.port, spi->pin_cs.num);
      return ARM_DRIVER_OK;

    default:
      return ARM_DRIVER_ERROR_UNSUPPORTED;
  }

  // Frame format:
  switch (control & ARM_SPI_FRAME_FORMAT_Msk) {
    case ARM_SPI_CPOL0_CPHA0:
      ctrl |= USART_CTRL_CLKPOL_IDLELOW      |
              USART_CTRL_CLKPHA_SAMPLELEADING;
      break;
    case ARM_SPI_CPOL0_CPHA1:
     ctrl |= USART_CTRL_CLKPOL_IDLELOW |
             USART_CTRL_CLKPHA_SAMPLETRAILING;
      break;
    case ARM_SPI_CPOL1_CPHA0:
      ctrl |= USART_CTRL_CLKPOL_IDLEHIGH |
              USART_CTRL_CLKPHA_SAMPLELEADING;
      break;
    case ARM_SPI_CPOL1_CPHA1:
      ctrl |= USART_CTRL_CLKPOL_IDLEHIGH|
              USART_CTRL_CLKPHA_SAMPLETRAILING;
      break;
    case ARM_SPI_TI_SSI:
      return ARM_SPI_ERROR_FRAME_FORMAT;
    case ARM_SPI_MICROWIRE:
      return ARM_SPI_ERROR_FRAME_FORMAT;
    default: return ARM_SPI_ERROR_FRAME_FORMAT;
  }

  // Data Bits
  val = (control & ARM_SPI_DATA_BITS_Msk) >> ARM_SPI_DATA_BITS_Pos;
  // Nine data bits transfer is ot supported
  if (val == 9) { return ARM_SPI_ERROR_DATA_BITS; }
  if ((val >= 4) && (val <= 16)) {
    frame |= (val - 3) << _USART_FRAME_DATABITS_SHIFT;
  } else return ARM_SPI_ERROR_DATA_BITS;

  // Bit order
  val = control & ARM_SPI_BIT_ORDER_Msk;
  if (val == 0U) { ctrl |= (1U << _USART_CTRL_MSBF_SHIFT); }

  // Slave select master modes
  if ((mode == ARM_SPI_MODE_MASTER) || (mode == ARM_SPI_MODE_MASTER_SIMPLEX)) {
    switch (control & ARM_SPI_SS_MASTER_MODE_Msk) {
      case ARM_SPI_SS_MASTER_UNUSED:
        if (spi->pin_cs_en) {
          spi->reg->ROUTE &= ~USART_ROUTE_CSPEN;
          GPIO_PinModeSet (spi->pin_cs.port, spi->pin_cs.num, gpioModeDisabled, 0);
        }
        mode |= ARM_SPI_SS_MASTER_UNUSED;
        break;

      case ARM_SPI_SS_MASTER_HW_INPUT:
        if (spi->pin_cs_en) {
          spi->reg->ROUTE |= USART_ROUTE_CSPEN;
          GPIO_PinModeSet (spi->pin_cs.port, spi->pin_cs.num, gpioModeInput, 0);
          ctrl |= USART_CTRL_CSMA;
        } else {
          return ARM_SPI_ERROR_SS_MODE;
        }
        mode |= ARM_SPI_SS_MASTER_HW_INPUT;
        break;

      case ARM_SPI_SS_MASTER_SW:
        if (spi->pin_cs_en) {
          spi->reg->ROUTE &= ~USART_ROUTE_CSPEN;
          GPIO_PinModeSet (spi->pin_cs.port, spi->pin_cs.num, gpioModePushPull, 1);
          mode |= ARM_SPI_SS_MASTER_SW;
        } else {
          return ARM_SPI_ERROR_SS_MODE;
        }
        break;

      case ARM_SPI_SS_MASTER_HW_OUTPUT:
        if (spi->pin_cs_en) {
          GPIO_PinModeSet (spi->pin_cs.port, spi->pin_cs.num, gpioModePushPull, 0);
          spi->reg->ROUTE |= USART_ROUTE_CSPEN;
          ctrl |= USART_CTRL_AUTOCS;
          mode |= ARM_SPI_SS_MASTER_HW_OUTPUT;
        } else {
          return ARM_SPI_ERROR_SS_MODE;
        }
        break;
    }
  }

  // Slave select slave modes
  if ((spi->info->mode ==  ARM_SPI_MODE_SLAVE) || (spi->info->mode == ARM_SPI_MODE_SLAVE_SIMPLEX)) {
    switch (control & ARM_SPI_SS_SLAVE_MODE_Msk) {
      case ARM_SPI_SS_SLAVE_HW:
        if (spi->pin_cs_en) {
          GPIO_PinModeSet (spi->pin_cs.port, spi->pin_cs.num, gpioModePushPull, 0);
          spi->reg->ROUTE |= USART_ROUTE_CSPEN;
          mode |= ARM_SPI_SS_SLAVE_HW;
        } else {
          return ARM_SPI_ERROR_SS_MODE;
        }
        break;

      case ARM_SPI_SS_SLAVE_SW:
        if (spi->pin_cs_en) {
          spi->reg->ROUTE &= ~USART_ROUTE_CSPEN;
          GPIO_PinModeSet (spi->pin_cs.port, spi->pin_cs.num, gpioModePushPull, 1);
          mode |= ARM_SPI_SS_SLAVE_SW;
        } else {
          return ARM_SPI_ERROR_SS_MODE;
        }
    }
  }

  // Set Baudrate
  if (spi->clock_limit && (arg > spi->clock_limit))
    arg = spi->clock_limit;
  USART_BaudrateSyncSet (spi->reg, 0, arg);

  spi->info->mode = mode;

  // Configure registers
  spi->reg->CMD   = USART_CMD_TXDIS | USART_CMD_RXDIS | USART_CMD_CLEARRX | USART_CMD_CLEARTX;
  spi->reg->FRAME = frame;
  spi->reg->CTRL  = ctrl;
  spi->reg->CMD   = cmd;
  spi->reg->IEN   = ien;

  if ((mode & ARM_SPI_CONTROL_Msk) == ARM_SPI_MODE_INACTIVE)
    spi->info->state &= ~SPI_CONFIGURED;
  else
    spi->info->state |=  SPI_CONFIGURED;

  // Enable transmiter and receiver
  spi->reg->CMD   = USART_CMD_TXEN | USART_CMD_RXEN;

  return ARM_DRIVER_OK;
}

/**
  \fn          ARM_SPI_STATUS SPI_GetStatus (SPI_RESOURCES *spi)
  \brief       Get SPI status.
  \param[in]   spi  Pointer to SPI resources
  \return      SPI status \ref ARM_SPI_STATUS
*/
static ARM_SPI_STATUS SPI_GetStatus (SPI_RESOURCES *spi) {
  ARM_SPI_STATUS status;

  status.busy       = spi->info->status.busy;
  status.data_lost  = spi->info->status.data_lost;
  status.mode_fault = spi->info->status.mode_fault;

  return (status);
}

/**
  \fn          void USART0_TX_IRQHandler (SPI_RESOURCES *spi)
  \brief       USART TX interrupt handler.
  \param[in]   spi  Pointer to SPI resources
*/
void USART_TX_IRQHandler (SPI_RESOURCES *spi) {
  uint16_t data;
  uint32_t if_reg, mode, event;

  event = 0U;
  if_reg = spi->reg->IF;

  if (if_reg & USART_IF_TXBL) {
    if (spi->xfer->tx_buf) {
      data = *(spi->xfer->tx_buf++);
      if ((spi->reg->FRAME & _USART_FRAME_DATABITS_MASK) > USART_FRAME_DATABITS_EIGHT)
        data |= (*(spi->xfer->tx_buf++) << 8);
    } else {
      data = spi->xfer->def_val;
    }

    if ((spi->reg->FRAME & _USART_FRAME_DATABITS_MASK) > USART_FRAME_DATABITS_NINE) {
      spi->reg->TXDOUBLE = data;
    } else {
      spi->reg->TXDATA = data;
    }
    spi->xfer->tx_cnt++;

    // Buffer empty or TX complete
    if (spi->xfer->num == spi->xfer->tx_cnt) {
      // Disable TX buffer empty interrupt
      spi->reg->IEN &= ~USART_IEN_TXBL;
      
      NVIC_ClearPendingIRQ(spi->usart_tx_irq);

      // If last RX interrupt is already processed, clear busy flag and
      // send transfer completed event
      if (spi->xfer->num == spi->xfer->rx_cnt) {
        spi->info->status.busy = 0;
        event |= ARM_SPI_EVENT_TRANSFER_COMPLETE;
      }
    }
  }

  mode = spi->info->mode & ARM_SPI_CONTROL_Msk;
  // Slave modes
  if ((mode == ARM_SPI_MODE_SLAVE) ||
      (mode == ARM_SPI_MODE_SLAVE_SIMPLEX)) {

    // Master has started transmission, when no buffer available
    if (if_reg & USART_IF_TXUF) {
      spi->info->status.data_lost = 1U;
      event |= ARM_SPI_EVENT_DATA_LOST;
    }
  }

  // Master modes
  if ((mode == ARM_SPI_MODE_MASTER) ||
      (mode == ARM_SPI_MODE_MASTER_SIMPLEX)) {

    if (if_reg & USART_IF_SSM) {
      // Device has been selected as slave, when in master mode
      if (mode == ARM_SPI_MODE_MASTER) mode = ARM_SPI_MODE_SLAVE;
      else                             mode = ARM_SPI_MODE_SLAVE_SIMPLEX;

      spi->reg->CMD = USART_CMD_TXDIS | USART_CMD_RXDIS | USART_CMD_CLEARRX | USART_CMD_CLEARTX;

      // Hardware slave select
      spi->reg->CTRL |= USART_CTRL_AUTOCS;

      // Disable master mode, enable transmitter and receiver
      spi->reg->CMD = USART_CMD_TXEN | USART_CMD_RXEN | USART_CMD_TXTRIDIS;

      // Enable TX underflow and RX overflow interrupts
      spi->reg->IEN |= USART_IEN_TXUF | USART_IEN_RXOF;

      spi->info->status.mode_fault = 1U;
      event |= ARM_SPI_EVENT_MODE_FAULT;
    }
  }

  if ((spi->info->cb_event != NULL) && (event != 0U)) {
    spi->info->cb_event (event);
  }
}

/**
  \fn          void USART0_RX_IRQHandler (SPI_RESOURCES *spi)
  \brief       USART RX interrupt handler.
  \param[in]   spi  Pointer to SPI resources
*/
void USART_RX_IRQHandler (SPI_RESOURCES *spi) {
  uint16_t data;
  uint32_t if_reg, mode, event;

  event = 0U;
  if_reg = spi->reg->IF;

  if (if_reg & USART_IF_RXDATAV) {
    if ((spi->reg->FRAME & _USART_FRAME_DATABITS_MASK) > USART_FRAME_DATABITS_EIGHT) {
      data = spi->reg->RXDOUBLE;
    } else {
      data = spi->reg->RXDATA;
    }

    // If buffer available, save data
    if (spi->xfer->rx_buf) {
      *(spi->xfer->rx_buf++) = (uint8_t)data;
      if ((spi->reg->FRAME & _USART_FRAME_DATABITS_MASK) > USART_FRAME_DATABITS_NINE) {
        *(spi->xfer->rx_buf++) = (uint8_t)(data >> 8);
      }
    }
    // Update rx counter
    spi->xfer->rx_cnt++;

    // Data available in the receive buffer
    if (spi->xfer->num == spi->xfer->rx_cnt) {
      // Disable RX data valid interrupt
      spi->reg->IEN &= ~USART_IF_RXDATAV;
      NVIC_ClearPendingIRQ(spi->usart_rx_irq);

      // If last TX interrupt is already processed, clear busy flag and
      // send transfer completed event
      if (spi->xfer->num == spi->xfer->tx_cnt) {
        spi->info->status.busy = 0U;
        event |= ARM_SPI_EVENT_TRANSFER_COMPLETE;
      }
    }
  }

  mode = spi->info->mode & ARM_SPI_CONTROL_Msk;
  // Slave modes
  if ((mode == ARM_SPI_MODE_SLAVE) ||
      (mode == ARM_SPI_MODE_SLAVE_SIMPLEX)) {

    // Master has started transmission, when no buffer available
    if (if_reg & USART_IF_RXOF) {
      spi->info->status.data_lost = 1U;
      event |= ARM_SPI_EVENT_DATA_LOST;
    }
  }

  if ((spi->info->cb_event != NULL) && (event != 0U)) {
    spi->info->cb_event (event);
  }
}

// DMA Callbacks
#if ((RTE_USART0 && RTE_USART0_SPI_MODE && RTE_USART0_DMA) | \
     (RTE_USART1 && RTE_USART1_SPI_MODE && RTE_USART1_DMA) | \
     (RTE_USART2 && RTE_USART2_SPI_MODE && RTE_USART2_DMA))
/**
  \fn          void USART_DMA_TxComplete (SPI_RESOURCES *spi)
  \brief       DMA transmit complete callback.
  \param[in]   spi  Pointer to SPI resources
*/
static void USART_DMA_TxComplete (SPI_RESOURCES *spi) {
  // TX completed
  if ((spi->xfer->tx_cnt + 1024) >= spi->xfer->num) {
    spi->xfer->tx_cnt = spi->xfer->num;
    if (spi->xfer->num == spi->xfer->rx_cnt) {
      spi->info->status.busy = 0;
      if (spi->info->cb_event) {
        spi->info->cb_event (ARM_SPI_EVENT_TRANSFER_COMPLETE);
      }
    }
  } else {
    spi->xfer->tx_cnt += 1024;
    if (spi->xfer->tx_cnt == spi->xfer->rx_cnt) {
      SPI_DMA_Transfer (spi);
    }
  }
}

/**
  \fn          void USART_DMA_RxComplete (SPI_RESOURCES *spi)
  \brief       DMA receive complete callback.
  \param[in]   spi  Pointer to SPI resources
*/
static void USART_DMA_RxComplete (SPI_RESOURCES *spi) {
  // RX completed
  if ((spi->xfer->rx_cnt + 1024) >= spi->xfer->num) {
    spi->xfer->rx_cnt = spi->xfer->num;
    if (spi->xfer->num == spi->xfer->tx_cnt) {
      spi->info->status.busy = 0;
      if (spi->info->cb_event)
        spi->info->cb_event (ARM_SPI_EVENT_TRANSFER_COMPLETE);
    }
  } else {
    spi->xfer->rx_cnt += 1024;
    if (spi->xfer->rx_cnt == spi->xfer->tx_cnt) {
      SPI_DMA_Transfer (spi);
    }
  }
}
#endif

/**
  \fn          void USART_DMA_Complete (uint32_t channel, bool primary, void *userPtr)
  \brief       DMA transaction complete callback.
  \param[in]   channel    Channel number
  \param[in]   primary    Primary descriptor
  \param[in]   userPtr    User defined pointer
*/
void USART_DMA_Complete (uint32_t channel, bool primary, void *userPtr) {
  #if (RTE_USART0 && RTE_USART0_SPI_MODE && RTE_USART0_DMA)
  if (channel == RTE_USART0_TX_DMA_CHANNEL) {
    USART_DMA_TxComplete (&SPI0_Resources);
  }
  if (channel == RTE_USART0_RX_DMA_CHANNEL) {
    USART_DMA_RxComplete (&SPI0_Resources);
  }
  #endif

  #if (RTE_USART1 && RTE_USART1_SPI_MODE && RTE_USART1_DMA)
  if (channel == RTE_USART1_TX_DMA_CHANNEL) {
    USART_DMA_TxComplete (&SPI1_Resources);
  }
  if (channel == RTE_USART1_RX_DMA_CHANNEL) {
    USART_DMA_RxComplete (&SPI1_Resources);
  }
  #endif

  #if (RTE_USART2 && RTE_USART2_SPI_MODE && RTE_USART2_DMA)
  if (channel == RTE_USART2_TX_DMA_CHANNEL) {
    USART_DMA_TxComplete (&SPI2_Resources);
  }
  if (channel == RTE_USART2_RX_DMA_CHANNEL) {
    USART_DMA_RxComplete (&SPI2_Resources);
  }
  #endif
}

// SPI0
#if (RTE_USART0 && RTE_USART0_SPI_MODE)
static int32_t        SPI0_Initialize          (ARM_SPI_SignalEvent_t pSignalEvent)                { return SPI_Initialize   (pSignalEvent, &SPI0_Resources); }
static int32_t        SPI0_Uninitialize        (void)                                              { return SPI_Uninitialize (&SPI0_Resources); }
static int32_t        SPI0_PowerControl        (ARM_POWER_STATE state)                             { return SPI_PowerControl (state, &SPI0_Resources); }
static int32_t        SPI0_Send                (const void *data, uint32_t num)                    { return SPI_Send         (data, num, &SPI0_Resources); }
static int32_t        SPI0_Receive             (void *data, uint32_t num)                          { return SPI_Receive      (data, num, &SPI0_Resources); }
static int32_t        SPI0_Transfer            (const void *data_out, void *data_in, uint32_t num) { return SPI_Transfer     (data_out, data_in, num, &SPI0_Resources); }
static uint32_t       SPI0_GetDataCount        (void)                                              { return SPI_GetDataCount (&SPI0_Resources); }
static int32_t        SPI0_Control             (uint32_t control, uint32_t arg)                    { return SPI_Control      (control, arg, &SPI0_Resources); }
static ARM_SPI_STATUS SPI0_GetStatus           (void)                                              { return SPI_GetStatus    (&SPI0_Resources); }
       void           USART0_TX_IRQHandler     (void)                                              { USART_TX_IRQHandler     (&SPI0_Resources); }
       void           USART0_RX_IRQHandler     (void)                                              { USART_RX_IRQHandler     (&SPI0_Resources); }

// SPI0 Driver Control Block
ARM_DRIVER_SPI Driver_SPI0 = {
  SPI_GetVersion,
  SPI_GetCapabilities,
  SPI0_Initialize,
  SPI0_Uninitialize,
  SPI0_PowerControl,
  SPI0_Send,
  SPI0_Receive,
  SPI0_Transfer,
  SPI0_GetDataCount,
  SPI0_Control,
  SPI0_GetStatus
};
#endif

// SPI1
#if (RTE_USART1 && RTE_USART1_SPI_MODE)
static int32_t        SPI1_Initialize          (ARM_SPI_SignalEvent_t pSignalEvent)                { return SPI_Initialize   (pSignalEvent, &SPI1_Resources); }
static int32_t        SPI1_Uninitialize        (void)                                              { return SPI_Uninitialize (&SPI1_Resources); }
static int32_t        SPI1_PowerControl        (ARM_POWER_STATE state)                             { return SPI_PowerControl (state, &SPI1_Resources); }
static int32_t        SPI1_Send                (const void *data, uint32_t num)                    { return SPI_Send         (data, num, &SPI1_Resources); }
static int32_t        SPI1_Receive             (void *data, uint32_t num)                          { return SPI_Receive      (data, num, &SPI1_Resources); }
static int32_t        SPI1_Transfer            (const void *data_out, void *data_in, uint32_t num) { return SPI_Transfer     (data_out, data_in, num, &SPI1_Resources); }
static uint32_t       SPI1_GetDataCount        (void)                                              { return SPI_GetDataCount (&SPI1_Resources); }
static int32_t        SPI1_Control             (uint32_t control, uint32_t arg)                    { return SPI_Control      (control, arg, &SPI1_Resources); }
static ARM_SPI_STATUS SPI1_GetStatus           (void)                                              { return SPI_GetStatus    (&SPI1_Resources); }
       void           USART1_TX_IRQHandler     (void)                                              { USART_TX_IRQHandler     (&SPI1_Resources); }
       void           USART1_RX_IRQHandler     (void)                                              { USART_RX_IRQHandler     (&SPI1_Resources); }

// SPI1 Driver Control Block
ARM_DRIVER_SPI Driver_SPI1 = {
  SPI_GetVersion,
  SPI_GetCapabilities,
  SPI1_Initialize,
  SPI1_Uninitialize,
  SPI1_PowerControl,
  SPI1_Send,
  SPI1_Receive,
  SPI1_Transfer,
  SPI1_GetDataCount,
  SPI1_Control,
  SPI1_GetStatus
};
#endif

// SPI2
#if (RTE_USART2 && RTE_USART2_SPI_MODE)
static int32_t        SPI2_Initialize          (ARM_SPI_SignalEvent_t pSignalEvent)                { return SPI_Initialize   (pSignalEvent, &SPI2_Resources); }
static int32_t        SPI2_Uninitialize        (void)                                              { return SPI_Uninitialize (&SPI2_Resources); }
static int32_t        SPI2_PowerControl        (ARM_POWER_STATE state)                             { return SPI_PowerControl (state, &SPI2_Resources); }
static int32_t        SPI2_Send                (const void *data, uint32_t num)                    { return SPI_Send         (data, num, &SPI2_Resources); }
static int32_t        SPI2_Receive             (void *data, uint32_t num)                          { return SPI_Receive      (data, num, &SPI2_Resources); }
static int32_t        SPI2_Transfer            (const void *data_out, void *data_in, uint32_t num) { return SPI_Transfer     (data_out, data_in, num, &SPI2_Resources); }
static uint32_t       SPI2_GetDataCount        (void)                                              { return SPI_GetDataCount (&SPI2_Resources); }
static int32_t        SPI2_Control             (uint32_t control, uint32_t arg)                    { return SPI_Control      (control, arg, &SPI2_Resources); }
static ARM_SPI_STATUS SPI2_GetStatus           (void)                                              { return SPI_GetStatus    (&SPI2_Resources); }
       void           USART2_TX_IRQHandler     (void)                                              { USART_TX_IRQHandler     (&SPI2_Resources); }
       void           USART2_RX_IRQHandler     (void)                                              { USART_RX_IRQHandler     (&SPI2_Resources); }

// SPI2 Driver Control Block
ARM_DRIVER_SPI Driver_SPI2 = {
  SPI_GetVersion,
  SPI_GetCapabilities,
  SPI2_Initialize,
  SPI2_Uninitialize,
  SPI2_PowerControl,
  SPI2_Send,
  SPI2_Receive,
  SPI2_Transfer,
  SPI2_GetDataCount,
  SPI2_Control,
  SPI2_GetStatus
};
#endif
