/* -----------------------------------------------------------------------------
 * Copyright (c) 2013-2017 ARM Ltd.
 *
 * This software is provided 'as-is', without any express or implied warranty.
 * In no event will the authors be held liable for any damages arising from
 * the use of this software. Permission is granted to anyone to use this
 * software for any purpose, including commercial applications, and to alter
 * it and redistribute it freely, subject to the following restrictions:
 *
 * 1. The origin of this software must not be misrepresented; you must not
 *    claim that you wrote the original software. If you use this software in
 *    a product, an acknowledgment in the product documentation would be
 *    appreciated but is not required.
 *
 * 2. Altered source versions must be plainly marked as such, and must not be
 *    misrepresented as being the original software.
 *
 * 3. This notice may not be removed or altered from any source distribution.
 *
 *
 * $Date:        19. September 2017
 * $Revision:    V2.6
 *
 * Driver:       Driver_USBH0
 * Configured:   via RTE_Device.h configuration file
 * Project:      USB Full/Low-Speed Host Driver for SiliconLabs EFM32GGxxx
 * --------------------------------------------------------------------------
 * Use the following configuration settings in the middleware component
 * to connect to this driver.
 *
 *   Configuration Setting                  Value
 *   ---------------------                  -----
 *   Connect to hardware via Driver_USBH# = 0
 *   USB Host controller interface        = Custom
 * --------------------------------------------------------------------------
 * Defines used for driver configuration (at compile time):
 *
 *   USBH_MAX_PIPE_NUM: defines maximum number of Pipes that driver will
 *                      support, this value impacts driver memory
 *                      requirements
 *     - default value: 14
 *     - maximum value: 14
 * -------------------------------------------------------------------------- */

/* History:
 *  Version 2.6
 *    Added support for CMSIS-RTOS2
 *  Version 2.5
 *    Removed interrupt priority handling
 *  Version 2.4
 *    PowerControl for Power OFF and Uninitialize functions made unconditional
 *  Version 2.3
 *    Corrected signalling of STALL handshake
 *  Version 2.2
 *    Controller init/deinit moved to PowerControl function
 *    Fixed interrupt pipe transfer interval and transfer done
 *    Fixed support for Transfers larger then FIFO size
 *  Version 2.1
 *    Added USBH state checking
 *  Version 2.0
 *    Initial release for USB Host Driver API v2.0
 *  Version 1.1
 *    Based on API V1.10 (namespace prefix ARM_ added)
 *  Version 1.0
 *    Initial release
 */


#include <stdint.h>
#include <string.h>

#include "RTE_Components.h"

#if       defined(RTE_CMSIS_RTOS2)
#include "cmsis_os2.h"
#elif     defined(RTE_CMSIS_RTOS)
#include "cmsis_os.h"
#endif

#include "em_device.h"

#include "em_cmu.h"
#include "em_emu.h"
#include "em_gpio.h"

#include "Driver_USBH.h"

#ifndef USBH_MAX_PIPE_NUM
#define USBH_MAX_PIPE_NUM               14U
#endif
#if    (USBH_MAX_PIPE_NUM > 14)
#error  Too many Pipes, maximum Pipes that this driver supports is 14 !!!
#endif

#define USBH_DRIVER_INITIALIZED (1U << 4)
#define USBH_DRIVER_POWERED     (1U << 5)

extern uint8_t usb_role;
extern uint8_t usb_drv_state;

extern void USB_PinsConfigure   (uint8_t pins_mask);
extern void USB_PinsUnconfigure (uint8_t pins_mask);
extern void USB_PinVbusOnOff    (bool state);
extern bool USB_PinGetOC        (void);


// USBH Driver *****************************************************************

#define ARM_USBH_DRV_VERSION ARM_DRIVER_VERSION_MAJOR_MINOR(2,6)

// Driver Version
static const ARM_DRIVER_VERSION usbh_driver_version = { ARM_USBH_API_VERSION, ARM_USBH_DRV_VERSION };

// Driver Capabilities
static const ARM_USBH_CAPABILITIES usbh_driver_capabilities = {
  0x0001U,      // Root HUB available Ports Mask
  0U,           // Automatic SPLIT packet handling
  1U,           // Signal Connect event
  1U,           // Signal Disconnect event
  0U            // Signal Overcurrent event
};

// FIFO sizes in bytes (total available memory for FIFOs is 2 kB)
#define RX_FIFO_SIZE            1024U   // RxFIFO depth is half of max 2 kB
#define TX_FIFO_SIZE_NON_PERI   512U    // Non-periodic Tx FIFO size
#define TX_FIFO_SIZE_PERI       512U    // Periodic Tx FIFO size

static volatile uint32_t *OTG_DFIFO[] = { (volatile uint32_t *) &(USB->FIFO0D),
                                          (volatile uint32_t *) &(USB->FIFO1D),
                                          (volatile uint32_t *) &(USB->FIFO2D),
                                          (volatile uint32_t *) &(USB->FIFO3D),
                                          (volatile uint32_t *) &(USB->FIFO4D),
                                          (volatile uint32_t *) &(USB->FIFO5D),
                                          (volatile uint32_t *) &(USB->FIFO6D),
                                          (volatile uint32_t *) &(USB->FIFO7D),
                                          (volatile uint32_t *) &(USB->FIFO8D),
                                          (volatile uint32_t *) &(USB->FIFO9D),
                                          (volatile uint32_t *) &(USB->FIFO10D),
                                          (volatile uint32_t *) &(USB->FIFO11D),
                                          (volatile uint32_t *) &(USB->FIFO12D),
                                          (volatile uint32_t *) &(USB->FIFO13D)
                                        };

typedef struct {                        // Pipe structure definition
  uint32_t  packet;
  uint8_t  *data;
  uint32_t  num;
  uint32_t  num_transferred_total;
  uint32_t  num_transferring;
  uint16_t  ep_max_packet_size;
  uint16_t  interval_reload;
  uint16_t  interval;
  uint8_t   ep_type;
  uint8_t   active;
  uint8_t   in_progress;
  uint8_t   event;
} PIPE_t;

static ARM_USBH_SignalPortEvent_t SignalPortEvent;
static ARM_USBH_SignalPipeEvent_t SignalPipeEvent;

static bool            port_reset;

// Pipes runtime information
static volatile PIPE_t pipe[USBH_MAX_PIPE_NUM];

// Auxiliary functions

/**
  \fn          uint32_t USBH_CH_GetIndexFromAddress (USB_HC_TypeDef *ptr_ch)
  \brief       Get the Index of Channel from it's Address.
  \param[in]   ptr_ch   Pointer to the Channel
  \return      Index of the Channel
*/
__INLINE static uint32_t USBH_CH_GetIndexFromAddress (USB_HC_TypeDef *ptr_ch) {
  return (ptr_ch - (USB_HC_TypeDef *)(&USB->HC[0].CHAR));
}

/**
  \fn          USB_HC_TypeDef *USBH_CH_GetAddressFromIndex (uint32_t index)
  \brief       Get the Channel Address from it's Index.
  \param[in]   index    Index of the Channel
  \return      Address of the Channel
*/
__INLINE static USB_HC_TypeDef *USBH_CH_GetAddressFromIndex (uint32_t index) {
  return ((USB_HC_TypeDef *)(&USB->HC[0].CHAR) + index);
}

/**
  \fn          USB_HC_TypeDef *USBH_CH_FindFree (void)
  \brief       Find a free Channel.
  \return      Pointer to the first free Channel (NULL = no free Channel is available)
*/
__INLINE static USB_HC_TypeDef *USBH_CH_FindFree (void) {
  USB_HC_TypeDef *ptr_ch;
  uint32_t        i;

  ptr_ch = (USB_HC_TypeDef *)(&USB->HC[0].CHAR);

  for (i = 0U; i < USBH_MAX_PIPE_NUM; i++) {
    if ((ptr_ch->CHAR & 0x3FFFFFFFU) == 0U) { return ptr_ch; }
    ptr_ch++;
  }

  return NULL;
}

/**
  \fn          bool USBH_CH_Disable (USB_HC_TypeDef *ptr_ch)
  \brief       Disable the Channel.
  \param[in]   ptr_ch   Pointer to the Channel
  \return      true = success, false = fail
*/
__INLINE static bool USBH_CH_Disable (USB_HC_TypeDef *ptr_ch) {
  uint32_t i;

  if (ptr_ch == 0U) { return false; }

  if ((ptr_ch->INT & USB_HC_INT_CHHLTD) != 0U) { return true; }

  if ((ptr_ch->CHAR & USB_HC_CHAR_CHENA) != 0U) {
    ptr_ch->INTMSK = 0U;
    osDelay(1U);
    if ((ptr_ch->CHAR & USB_HC_INT_NAK) != 0U) {
      ptr_ch->INT  =  0x7BBU;
      return true;
    }
    ptr_ch->INT  =  0x7BBU;
    ptr_ch->CHAR =  ptr_ch->CHAR | USB_HC_CHAR_CHENA | USB_HC_CHAR_CHDIS;
    for (i = 0U; i < 1000U; i++) {
      if ((ptr_ch->INT & USB_HC_INT_CHHLTD) != 0U) {
        ptr_ch->INT = 0x7BBU;
        return true;
      }
    }
    return false;
  }

  return true;
}

/**
  \fn          bool USBH_HW_StartTransfer (PIPE_t *ptr_pipe, USB_HC_TypeDef *ptr_ch)
  \brief       Start transfer on Pipe.
  \param[in]   ptr_pipe Pointer to Pipe
  \param[in]   ptr_ch   Pointer to Channel
  \return      true = success, false = fail
*/
static bool USBH_HW_StartTransfer (PIPE_t *ptr_pipe, USB_HC_TypeDef *ptr_ch) {
  uint32_t           hcchar;
  uint32_t           hctsiz;
  uint32_t           hcintmsk;
  uint32_t           num_to_transfer;
  uint8_t           *ptr_src;
  volatile uint32_t *ptr_dest;
  uint16_t           cnt;
  uint8_t            out;

  if (ptr_pipe == 0U)                          { return false; }
  if (ptr_ch   == 0U)                          { return false; }
  if ((USB->HPRT & USB_HPRT_PRTCONNSTS) == 0U) { return false; }

  hcchar   = ptr_ch->CHAR;                      // Read channel characteristics
  hctsiz   = ptr_ch->TSIZ;                      // Read channel size info
  hcintmsk = 0U;
  cnt      = 0U;
  out      = 0U;

  // Prepare transfer
                                                // Prepare HCCHAR register
  hcchar &=       (USB_HC_CHAR_ODDFRM         | // Keep ODDFRM
                  _USB_HC_CHAR_DEVADDR_MASK   | // Keep DEVADDR
                  _USB_HC_CHAR_MC_MASK        | // Keep MC
                  _USB_HC_CHAR_EPTYPE_MASK    | // Keep EPTYPE
                   USB_HC_CHAR_LSPDDEV        | // Keep LSPDDEV
                  _USB_HC_CHAR_EPNUM_MASK     | // Keep EPNUM
                  _USB_HC_CHAR_MPS_MASK)      ; // Keep MPS
  switch (ptr_pipe->packet & ARM_USBH_PACKET_TOKEN_Msk) {
    case ARM_USBH_PACKET_IN:
      hcchar   |= _USB_HC_CHAR_EPDIR_MASK;
      hcintmsk  = (USB_HC_INTMSK_DATATGLERRMSK|
                   USB_HC_INTMSK_BBLERRMSK    |
                   USB_HC_INTMSK_XACTERRMSK   |
                   USB_HC_INTMSK_ACKMSK       |
                   USB_HC_INTMSK_NAKMSK       |
                   USB_HC_INTMSK_STALLMSK     |
                   USB_HC_INTMSK_XFERCOMPLMSK);
      break;
    case ARM_USBH_PACKET_OUT:
      hcchar   &= ~_USB_HC_CHAR_EPDIR_MASK;
      hcintmsk  = (USB_HC_INTMSK_XACTERRMSK   |
                   USB_HC_INTMSK_ACKMSK       |
                   USB_HC_INTMSK_NAKMSK       |
                   USB_HC_INTMSK_STALLMSK     |
                   USB_HC_INTMSK_XFERCOMPLMSK);
      out       =  1U;
      break;
    case ARM_USBH_PACKET_SETUP:
      hcchar   &= ~_USB_HC_CHAR_EPDIR_MASK;
      hcintmsk  = (USB_HC_INTMSK_XACTERRMSK   |
                   USB_HC_INTMSK_XFERCOMPLMSK);
      hctsiz   &= ~_USB_HC_TSIZ_PID_MASK;
      hctsiz   |=  USB_HC_TSIZ_PID_MDATA;
      out       =  1U;
      break;
    default:
      return false;
  }

  num_to_transfer = ptr_pipe->num - ptr_pipe->num_transferred_total;
  switch (ptr_pipe->ep_type) {
    case ARM_USB_ENDPOINT_CONTROL:
    case ARM_USB_ENDPOINT_BULK:
      if (out != 0U) {
        // For packet OUT/SETUP limit number of bytes to send to maximum FIFO size
        if (num_to_transfer > TX_FIFO_SIZE_NON_PERI) {
          num_to_transfer = TX_FIFO_SIZE_NON_PERI;
        }
        cnt = (num_to_transfer + 3U) / 4U;
      }
      break;
    case ARM_USB_ENDPOINT_ISOCHRONOUS:
    case ARM_USB_ENDPOINT_INTERRUPT:
      if (out != 0U) {
        // For packet OUT/SETUP limit number of bytes to send to maximum FIFO size
        if (num_to_transfer > TX_FIFO_SIZE_PERI) {
          num_to_transfer = TX_FIFO_SIZE_PERI;
        }
        cnt = (num_to_transfer + 3U) / 4U;
      }
      if ((USB->HFNUM & 1U) != 0U) {
        hcchar &= ~USB_HC_CHAR_ODDFRM;
      } else {
        hcchar |=  USB_HC_CHAR_ODDFRM;
      }
      break;
  }
  hcchar       &=  ~USB_HC_CHAR_CHDIS;
  hcchar       |=   USB_HC_CHAR_CHENA;

                                                  // Prepare TSIZ register
  hctsiz       &=  _USB_HC_TSIZ_PID_MASK;         // Keep PID
  switch (ptr_pipe->packet & ARM_USBH_PACKET_DATA_Msk) {
    case ARM_USBH_PACKET_DATA0:
      hctsiz   &= ~_USB_HC_TSIZ_PID_MASK;
      hctsiz   |=   USB_HC_TSIZ_PID_DATA0;
      break;
    case ARM_USBH_PACKET_DATA1:
      hctsiz   &= ~_USB_HC_TSIZ_PID_MASK;
      hctsiz   |=   USB_HC_TSIZ_PID_DATA1;
      break;
    default:
      break;
  }

                                                  // Prepare HCTSIZ register
  if (num_to_transfer != 0U) {                    // Normal packet
                                                  // Prepare PKTCNT field
    hctsiz |= ((num_to_transfer + ptr_pipe->ep_max_packet_size - 1U) / ptr_pipe->ep_max_packet_size) << 19;
    hctsiz |=   num_to_transfer;                  // Prepare XFERSIZE field
  } else {                                        // Zero length packet
    hctsiz |=   1U << 19;                         // Prepare PKTCNT field
    hctsiz |=   0U;                               // Prepare XFERSIZE field
  }

  if (cnt != 0U) {
    ptr_src  = ptr_pipe->data + ptr_pipe->num_transferred_total;
    ptr_dest = OTG_DFIFO[USBH_CH_GetIndexFromAddress (ptr_ch)];
  }
  if (out != 0U) {
    // For OUT/SETUP transfer num_transferring represents num of bytes to be sent
    ptr_pipe->num_transferring = num_to_transfer;
  } else {
    // For IN transfer num_transferring represents num of bytes received (handled in IRQ)
    ptr_pipe->num_transferring = 0U;
  }
  NVIC_DisableIRQ (USB_IRQn);                     // Disable USB interrupt
  ptr_ch->INTMSK = hcintmsk;                      // Enable channel interrupts
  ptr_ch->TSIZ   = hctsiz;                        // Write ch transfer size
  ptr_ch->CHAR   = hcchar;                        // Write ch characteristics
  while (cnt != 0U) {                             // Load data
    *ptr_dest = *((__packed uint32_t *)ptr_src);
    ptr_src  += 4U;
    cnt--;
  }
  NVIC_EnableIRQ  (USB_IRQn);                     // Enable USB interrupt

  return true;
}


// USBH Driver functions

/**
  \fn          ARM_DRIVER_VERSION USBH_GetVersion (void)
  \brief       Get driver version.
  \return      \ref ARM_DRIVER_VERSION
*/
static ARM_DRIVER_VERSION USBH_GetVersion (void) { return usbh_driver_version; }

/**
  \fn          ARM_USBH_CAPABILITIES USBH_GetCapabilities (void)
  \brief       Get driver capabilities.
  \return      \ref ARM_USBH_CAPABILITIES
*/
static ARM_USBH_CAPABILITIES USBH_GetCapabilities (void) { return usbh_driver_capabilities; }

/**
  \fn          int32_t USBH_Initialize (ARM_USBH_SignalPortEvent_t cb_port_event,
                                        ARM_USBH_SignalPipeEvent_t cb_pipe_event)
  \brief       Initialize USB Host Interface.
  \param[in]   cb_port_event  Pointer to \ref ARM_USBH_SignalPortEvent
  \param[in]   cb_pipe_event  Pointer to \ref ARM_USBH_SignalPipeEvent
  \return      \ref execution_status
*/
static int32_t USBH_Initialize (ARM_USBH_SignalPortEvent_t cb_port_event,
                                ARM_USBH_SignalPipeEvent_t cb_pipe_event) {

  if ((usb_drv_state & USBH_DRIVER_INITIALIZED) != 0U) { return ARM_DRIVER_OK; }

  SignalPortEvent =  cb_port_event;
  SignalPipeEvent =  cb_pipe_event;

  usb_role        =  ARM_USB_ROLE_HOST;
  USB_PinsConfigure (ARM_USB_PIN_DP | ARM_USB_PIN_DM | ARM_USB_PIN_OC | ARM_USB_PIN_VBUS);

  usb_drv_state   =  USBH_DRIVER_INITIALIZED;

  return ARM_DRIVER_OK;
}

/**
  \fn          int32_t USBH_Uninitialize (void)
  \brief       De-initialize USB Host Interface.
  \return      \ref execution_status
*/
static int32_t USBH_Uninitialize (void) {

  USB_PinsUnconfigure (ARM_USB_PIN_DP | ARM_USB_PIN_DM | ARM_USB_PIN_OC | ARM_USB_PIN_VBUS);
  usb_role        =  ARM_USB_ROLE_NONE;
  usb_drv_state  &= ~USBH_DRIVER_INITIALIZED;

  return ARM_DRIVER_OK;
}

/**
  \fn          int32_t USBH_PowerControl (ARM_POWER_STATE state)
  \brief       Control USB Host Interface Power.
  \param[in]   state  Power state
  \return      \ref execution_status
*/
static int32_t USBH_PowerControl (ARM_POWER_STATE state) {

  switch (state) {
    case ARM_POWER_OFF:
      NVIC_DisableIRQ      (USB_IRQn);                  // Disable interrupt
      NVIC_ClearPendingIRQ (USB_IRQn);                  // Clear pending interrupt
      usb_drv_state &= ~USBH_DRIVER_POWERED;            // Clear powered flag
      USB->GAHBCFG  &= ~USB_GAHBCFG_GLBLINTRMSK;        // Disable USB interrupts
                                                        // Reset variables
      port_reset =  false;
      memset((void *)pipe, 0, sizeof(pipe));
      // Disable USB clock
      CMU->HFCORECLKEN0 &= ~(CMU_HFCORECLKEN0_USB | CMU_HFCORECLKEN0_USBC);
      break;

    case ARM_POWER_FULL:
      if ((usb_drv_state & USBH_DRIVER_POWERED) != 0U) { return ARM_DRIVER_OK; }

      CMU->HFCORECLKEN0 |= CMU_HFCORECLKEN0_USBC;
      CMU_ClockSelectSet (cmuClock_USBC, cmuSelect_HFCLK);
      CMU->HFCORECLKEN0 |= CMU_HFCORECLKEN0_USB;

      // Wait until AHB Master state machine is in the idle condition
      while ((USB->GRSTCTL & USB_GRSTCTL_AHBIDLE) == 0U);

      USB->GRSTCTL  |=  USB_GRSTCTL_CSFTRST;            // Core soft reset
      while ((USB->GRSTCTL & USB_GRSTCTL_CSFTRST) != 0U);
      osDelay(1U);

      // Wait until AHB Master state machine is in the idle condition
      while ((USB->GRSTCTL & USB_GRSTCTL_AHBIDLE) == 0U);

                                                        // Reset variables
      port_reset =  false;
      memset((void *)(pipe), 0, sizeof(pipe));

      USB->GUSBCFG   &= ~USB_GUSBCFG_FSINTF;            // Full-speed transceiver

      if (((USB->GUSBCFG & USB_GUSBCFG_FORCEHSTMODE) == 0U) || ((USB->GUSBCFG & USB_GUSBCFG_FORCEDEVMODE) == 1U)) {
        USB->GUSBCFG &= ~USB_GUSBCFG_FORCEDEVMODE;
        USB->GUSBCFG |=  USB_GUSBCFG_FORCEHSTMODE;
        osDelay(50U);
      }

      // Rx FIFO setting
      USB->GRXFSIZ   = (RX_FIFO_SIZE / 4U);
      // Non-periodic Tx FIFO setting
      USB->GNPTXFSIZ = ((TX_FIFO_SIZE_NON_PERI / 4U) << 16) |  (RX_FIFO_SIZE / 4U);
      // Periodic Tx FIFO setting
      USB->HPTXFSIZ  = ((TX_FIFO_SIZE_PERI     / 4U) << 16) | ((RX_FIFO_SIZE + TX_FIFO_SIZE_NON_PERI) / 4U);

      USB->HAINTMSK  = (1U << USBH_MAX_PIPE_NUM) - 1U;  // Enable channel interrupts
      USB->GINTMSK   = (USB_GINTMSK_DISCONNINTMSK |     // Unmask interrupts
                        USB_GINTMSK_HCHINTMSK     |
                        USB_GINTMSK_PRTINTMSK     |
                        USB_GINTMSK_RXFLVLMSK     |
                        USB_GINTMSK_SOFMSK)       ;

      USB->GAHBCFG  |=  USB_GAHBCFG_GLBLINTRMSK;        // Enable USB interrupts

      usb_drv_state |=  USBH_DRIVER_POWERED;            // Set powered flag
      NVIC_EnableIRQ       (USB_IRQn);                  // Enable interrupt
      break;

    default:
      return ARM_DRIVER_ERROR_UNSUPPORTED;
  }

  return ARM_DRIVER_OK;
}

/**
  \fn          int32_t USBH_PortVbusOnOff (uint8_t port, bool vbus)
  \brief       Root HUB Port VBUS on/off.
  \param[in]   port  Root HUB Port Number
  \param[in]   vbus
                - \b false VBUS off
                - \b true  VBUS on
  \return      \ref execution_status
*/
static int32_t USBH_PortVbusOnOff (uint8_t port, bool vbus) {

  if ((usb_drv_state & USBH_DRIVER_POWERED) == 0U) { return ARM_DRIVER_ERROR;           }
  if (port != 0U)                                  { return ARM_DRIVER_ERROR_PARAMETER; }

  if (vbus != 0U) {                                     // VBUS power on
    USB->HPRT    |=  USB_HPRT_PRTPWR;                   // Port power on
    USB_PinVbusOnOff (true);                            // VBUS pin on
  } else {                                              // VBUS power off
    USB_PinVbusOnOff (false);                           // VBUS pin off
    USB->HPRT    &= ~USB_HPRT_PRTPWR;                   // Port power off
  }

  return ARM_DRIVER_OK;
}

/**
  \fn          int32_t USBH_PortReset (uint8_t port)
  \brief       Do Root HUB Port Reset.
  \param[in]   port  Root HUB Port Number
  \return      \ref execution_status
*/
static int32_t USBH_PortReset (uint8_t port) {
  uint32_t hprt;
  uint32_t hcfg;

  if ((usb_drv_state & USBH_DRIVER_POWERED) == 0U) { return ARM_DRIVER_ERROR;           }
  if (port != 0U)                                  { return ARM_DRIVER_ERROR_PARAMETER; }

  hcfg = USB->HCFG;
  hprt = USB->HPRT;
  switch ((hprt >> 17) & 3U) {
    case 0:                                             // High-speed detected
      break;
    case 1:                                             // Full-speed detected
      if (USB->HFIR != 48000U) { USB->HFIR = 48000U; }
      if ((hcfg & 3U) != 1U) {
        USB->HCFG = (hcfg & ~_USB_HCFG_FSLSPCLKSEL_MASK) | _USB_HCFG_FSLSPCLKSEL_DIV1;
      }
      break;
    case 2:                                             // Low-speed detected
      if (USB->HFIR != 6000U)  { USB->HFIR = 6000U;  }
      if ((hcfg & 3U) != 2U) {
        USB->HCFG = (hcfg & ~_USB_HCFG_FSLSPCLKSEL_MASK) | _USB_HCFG_FSLSPCLKSEL_DIV8;
      }
      break;
    case 3:
      break;
  }

  port_reset = true;
  hprt  =  USB->HPRT;
  hprt &= ~USB_HPRT_PRTENA;                             // Disable port
  hprt |=  USB_HPRT_PRTRST;                             // Port reset
  USB->HPRT = hprt;
  osDelay(20U);
  hprt &= ~USB_HPRT_PRTRST;                             // Clear port reset
  USB->HPRT = hprt;
  osDelay(50U);
  if (port_reset != false) {
    port_reset = false;
    return ARM_DRIVER_ERROR;
  }

  return ARM_DRIVER_OK;
}

/**
  \fn          int32_t USBH_PortSuspend (uint8_t port)
  \brief       Suspend Root HUB Port (stop generating SOFs).
  \param[in]   port  Root HUB Port Number
  \return      \ref execution_status
*/
static int32_t USBH_PortSuspend (uint8_t port) {

  if ((usb_drv_state & USBH_DRIVER_POWERED) == 0U) { return ARM_DRIVER_ERROR;           }
  if (port != 0U)                                  { return ARM_DRIVER_ERROR_PARAMETER; }

  USB->HPRT |=  USB_HPRT_PRTSUSP;                       // Port suspend

  return ARM_DRIVER_OK;
}

/**
  \fn          int32_t USBH_PortResume (uint8_t port)
  \brief       Resume Root HUB Port (start generating SOFs).
  \param[in]   port  Root HUB Port Number
  \return      \ref execution_status
*/
static int32_t USBH_PortResume (uint8_t port) {

  if ((usb_drv_state & USBH_DRIVER_POWERED) == 0U) { return ARM_DRIVER_ERROR;           }
  if (port != 0U)                                  { return ARM_DRIVER_ERROR_PARAMETER; }

  USB->HPRT |=  USB_HPRT_PRTRES;                        // Port resume

  return ARM_DRIVER_OK;
}

/**
  \fn          ARM_USBH_PORT_STATE USBH_PortGetState (uint8_t port)
  \brief       Get current Root HUB Port State.
  \param[in]   port  Root HUB Port Number
  \return      Port State \ref ARM_USBH_PORT_STATE
*/
static ARM_USBH_PORT_STATE USBH_PortGetState (uint8_t port) {
  ARM_USBH_PORT_STATE port_state = { 0U, 0U, 0U };
  uint32_t hprt;

  if ((usb_drv_state & USBH_DRIVER_POWERED) == 0U) { return port_state; }
  if (port != 0U)                                  { return port_state; }

  hprt = USB->HPRT;

  port_state.connected   = (hprt & USB_HPRT_PRTCONNSTS) != 0U;
  port_state.overcurrent = USB_PinGetOC ();
  switch ((hprt & _USB_HPRT_PRTSPD_MASK) >> _USB_HPRT_PRTSPD_SHIFT) {
    case _USB_HPRT_PRTSPD_HS:                           // High speed
     break;
    case _USB_HPRT_PRTSPD_FS:                           // Full speed
     port_state.speed = ARM_USB_SPEED_FULL;
     break;
    case _USB_HPRT_PRTSPD_LS:                           // Low speed
     port_state.speed = ARM_USB_SPEED_LOW;
     break;
    default:
     break;
  }

  return port_state;
}

/**
  \fn          ARM_USBH_PIPE_HANDLE USBH_PipeCreate (uint8_t  dev_addr,
                                                     uint8_t  dev_speed,
                                                     uint8_t  hub_addr,
                                                     uint8_t  hub_port,
                                                     uint8_t  ep_addr,
                                                     uint8_t  ep_type,
                                                     uint16_t ep_max_packet_size,
                                                     uint8_t  ep_interval)
  \brief       Create Pipe in System.
  \param[in]   dev_addr   Device Address
  \param[in]   dev_speed  Device Speed
  \param[in]   hub_addr   Hub Address
  \param[in]   hub_port   Hub Port
  \param[in]   ep_addr    Endpoint Address
                - ep_addr.0..3: Address
                - ep_addr.7:    Direction
  \param[in]   ep_type    Endpoint Type (ARM_USB_ENDPOINT_xxx)
  \param[in]   ep_max_packet_size Endpoint Maximum Packet Size
  \param[in]   ep_interval        Endpoint Polling Interval
  \return      Pipe Handle \ref ARM_USBH_PIPE_HANDLE
*/
static ARM_USBH_PIPE_HANDLE USBH_PipeCreate (uint8_t dev_addr, uint8_t dev_speed, uint8_t hub_addr, uint8_t hub_port, uint8_t ep_addr, uint8_t ep_type, uint16_t ep_max_packet_size, uint8_t  ep_interval) {
  PIPE_t         *ptr_pipe;
  USB_HC_TypeDef *ptr_ch;

  if ((usb_drv_state & USBH_DRIVER_POWERED) == 0U) { return 0U; }

  ptr_ch = USBH_CH_FindFree ();                 // Find free Channel
  if (ptr_ch == 0U) { return 0U; }              // If no free

  ptr_pipe = (PIPE_t *)(&pipe[USBH_CH_GetIndexFromAddress (ptr_ch)]);

  memset((void *)ptr_pipe, 0, sizeof(PIPE_t));  // Clear Pipes runtime information

  // Fill in all fields of Endpoint Descriptor
  ptr_ch->CHAR = (((uint32_t)ep_max_packet_size     << _USB_HC_CHAR_MPS_SHIFT    ) & _USB_HC_CHAR_MPS_MASK    ) |
                 (((uint32_t)ep_addr                << _USB_HC_CHAR_EPNUM_SHIFT  ) & _USB_HC_CHAR_EPNUM_MASK  ) |
                 (((((uint32_t)ep_addr >> 7) & 1U)  << _USB_HC_CHAR_EPDIR_SHIFT  ) & _USB_HC_CHAR_EPDIR_MASK  ) |
                 (((dev_speed == ARM_USB_SPEED_LOW) << _USB_HC_CHAR_LSPDDEV_SHIFT) & _USB_HC_CHAR_LSPDDEV_MASK) |
                 (((uint32_t)ep_type                << _USB_HC_CHAR_EPTYPE_SHIFT ) & _USB_HC_CHAR_EPTYPE_MASK ) |
                 (((uint32_t)dev_addr               << _USB_HC_CHAR_DEVADDR_SHIFT) & _USB_HC_CHAR_DEVADDR_MASK) ;

  // Store Pipe settings
  ptr_pipe->ep_max_packet_size = ep_max_packet_size;
  ptr_pipe->ep_type            = ep_type;
  switch (ep_type) {
    case ARM_USB_ENDPOINT_CONTROL:
    case ARM_USB_ENDPOINT_BULK:
      break;
    case ARM_USB_ENDPOINT_ISOCHRONOUS:
    case ARM_USB_ENDPOINT_INTERRUPT:
      if (dev_speed == ARM_USB_SPEED_HIGH) {
        if ((ep_interval > 0U) && (ep_interval <= 16U)) {
          ptr_pipe->interval_reload = (uint16_t)1U << (ep_interval - 1U);
        }
      } else if ((dev_speed == ARM_USB_SPEED_FULL) || (dev_speed == ARM_USB_SPEED_LOW)) {
        if (ep_interval > 0U) {
          ptr_pipe->interval_reload = ep_interval;
        }
      }
      ptr_pipe->interval = 0U;
      ptr_ch->CHAR |= (((((uint32_t)ep_max_packet_size >> 11) + 1U) & 3U) << _USB_HC_CHAR_MC_SHIFT) & _USB_HC_CHAR_MC_MASK;
      break;
  }

  return ((ARM_USBH_EP_HANDLE)ptr_ch);
}

/**
  \fn          int32_t USBH_PipeModify (ARM_USBH_PIPE_HANDLE pipe_hndl,
                                        uint8_t              dev_addr,
                                        uint8_t              dev_speed,
                                        uint8_t              hub_addr,
                                        uint8_t              hub_port,
                                        uint16_t             ep_max_packet_size)
  \brief       Modify Pipe in System.
  \param[in]   pipe_hndl  Pipe Handle
  \param[in]   dev_addr   Device Address
  \param[in]   dev_speed  Device Speed
  \param[in]   hub_addr   Hub Address
  \param[in]   hub_port   Hub Port
  \param[in]   ep_max_packet_size Endpoint Maximum Packet Size
  \return      \ref execution_status
*/
static int32_t USBH_PipeModify (ARM_USBH_PIPE_HANDLE pipe_hndl, uint8_t dev_addr, uint8_t dev_speed, uint8_t hub_addr, uint8_t hub_port, uint16_t ep_max_packet_size) {
  PIPE_t         *ptr_pipe;
  USB_HC_TypeDef *ptr_ch;
  uint32_t        hcchar;

  if ((usb_drv_state & USBH_DRIVER_POWERED) == 0U) { return ARM_DRIVER_ERROR;           }
  if (pipe_hndl == 0U)                             { return ARM_DRIVER_ERROR_PARAMETER; }

  ptr_ch   = (USB_HC_TypeDef *)(pipe_hndl);
  ptr_pipe = (PIPE_t         *)(&pipe[USBH_CH_GetIndexFromAddress (ptr_ch)]);
  if (ptr_pipe->active != 0U)                      { return ARM_DRIVER_ERROR_BUSY;      }

  // Fill in all fields of Endpoint Descriptor
  hcchar  =   ptr_ch->CHAR;
  hcchar &= ~(_USB_HC_CHAR_MPS_MASK      |              // Clear maximum packet size field
              _USB_HC_CHAR_LSPDDEV_MASK  |              // Clear device speed bit
              _USB_HC_CHAR_DEVADDR_MASK) ;              // Clear device address field
  hcchar |= (((uint32_t)ep_max_packet_size     << _USB_HC_CHAR_MPS_SHIFT)     & _USB_HC_CHAR_MPS_MASK    ) |
            (((dev_speed == ARM_USB_SPEED_LOW) << _USB_HC_CHAR_LSPDDEV_SHIFT) & _USB_HC_CHAR_LSPDDEV_MASK) |
            ((((uint32_t)dev_addr              << _USB_HC_CHAR_DEVADDR_SHIFT) & _USB_HC_CHAR_DEVADDR_MASK));
  ptr_ch->CHAR = hcchar;                                // Update modified fields

  ptr_pipe->ep_max_packet_size = ep_max_packet_size;

  return ARM_DRIVER_OK;
}

/**
  \fn          int32_t USBH_PipeDelete (ARM_USBH_PIPE_HANDLE pipe_hndl)
  \brief       Delete Pipe from System.
  \param[in]   pipe_hndl  Pipe Handle
  \return      \ref execution_status
*/
static int32_t USBH_PipeDelete (ARM_USBH_PIPE_HANDLE pipe_hndl) {
  PIPE_t         *ptr_pipe;
  USB_HC_TypeDef *ptr_ch;

  if ((usb_drv_state & USBH_DRIVER_POWERED) == 0U) { return ARM_DRIVER_ERROR;           }
  if (pipe_hndl == 0U)                             { return ARM_DRIVER_ERROR_PARAMETER; }

  ptr_ch   = (USB_HC_TypeDef *)(pipe_hndl);
  ptr_pipe = (PIPE_t         *)(&pipe[USBH_CH_GetIndexFromAddress (ptr_ch)]);
  if (ptr_pipe->active != 0U)                      { return ARM_DRIVER_ERROR_BUSY;      }

  ptr_ch->CHAR   = 0U;
  ptr_ch->INT    = 0U;
  ptr_ch->INTMSK = 0U;
  ptr_ch->TSIZ   = 0U;

  // Clear all fields of Pipe structure
  memset((void *)ptr_pipe, 0, sizeof(PIPE_t));

  return ARM_DRIVER_OK;
}

/**
  \fn          int32_t USBH_PipeReset (ARM_USBH_PIPE_HANDLE pipe_hndl)
  \brief       Reset Pipe.
  \param[in]   pipe_hndl  Pipe Handle
  \return      \ref execution_status
*/
static int32_t USBH_PipeReset (ARM_USBH_PIPE_HANDLE pipe_hndl) {
  PIPE_t         *ptr_pipe;
  USB_HC_TypeDef *ptr_ch;

  if ((usb_drv_state & USBH_DRIVER_POWERED) == 0U) { return ARM_DRIVER_ERROR;           }
  if (pipe_hndl == 0U)                             { return ARM_DRIVER_ERROR_PARAMETER; }

  ptr_ch   = (USB_HC_TypeDef *)(pipe_hndl);
  ptr_pipe = (PIPE_t         *)(&pipe[USBH_CH_GetIndexFromAddress (ptr_ch)]);
  if (ptr_pipe->active != 0U)                      { return ARM_DRIVER_ERROR_BUSY;      }

  ptr_ch->INT    = 0U;
  ptr_ch->INTMSK = 0U;
  ptr_ch->TSIZ   = 0U;

  return ARM_DRIVER_OK;
}

/**
  \fn          int32_t USBH_PipeTransfer (ARM_USBH_PIPE_HANDLE pipe_hndl,
                                          uint32_t             packet,
                                          uint8_t             *data,
                                          uint32_t             num)
  \brief       Transfer packets through USB Pipe.
  \param[in]   pipe_hndl  Pipe Handle
  \param[in]   packet     Packet information
  \param[in]   data       Pointer to buffer with data to send or for data to receive
  \param[in]   num        Number of data bytes to transfer
  \return      \ref execution_status
*/
static int32_t USBH_PipeTransfer (ARM_USBH_PIPE_HANDLE pipe_hndl, uint32_t packet, uint8_t *data, uint32_t num) {
  PIPE_t *ptr_pipe;

  if ((usb_drv_state & USBH_DRIVER_POWERED) == 0U) { return ARM_DRIVER_ERROR;           }
  if (pipe_hndl == 0U)                             { return ARM_DRIVER_ERROR_PARAMETER; }
  if ((USB->HPRT & USB_HPRT_PRTCONNSTS) == 0U)     { return ARM_DRIVER_ERROR;           }

  ptr_pipe = (PIPE_t *)(&pipe[USBH_CH_GetIndexFromAddress ((USB_HC_TypeDef *)(pipe_hndl))]);
  if (ptr_pipe->active != 0U)                      { return ARM_DRIVER_ERROR_BUSY;      }

  // Update current transfer information
  ptr_pipe->packet                = packet;
  ptr_pipe->data                  = data;
  ptr_pipe->num                   = num;
  ptr_pipe->num_transferred_total = 0U;
  ptr_pipe->num_transferring      = 0U;
  ptr_pipe->active                = 0U;
  ptr_pipe->in_progress           = 0U;
  ptr_pipe->event                 = 0U;
  if ((ptr_pipe->ep_type == ARM_USB_ENDPOINT_INTERRUPT) && (ptr_pipe->interval != 0U)) {
    // Already active interrupt endpoint (it will restart in IRQ based on interval)
    ptr_pipe->active              = 1U;
  } else {
    ptr_pipe->in_progress         = 1U;
    ptr_pipe->active              = 1U;
    if (USBH_HW_StartTransfer (ptr_pipe, (USB_HC_TypeDef *)pipe_hndl) == 0U) {
      ptr_pipe->in_progress       = 0U;
      ptr_pipe->active            = 0U;
      return ARM_DRIVER_ERROR;
    }
  }

  return ARM_DRIVER_OK;
}

/**
  \fn          uint32_t USBH_PipeTransferGetResult (ARM_USBH_PIPE_HANDLE pipe_hndl)
  \brief       Get result of USB Pipe transfer.
  \param[in]   pipe_hndl  Pipe Handle
  \return      number of successfully transferred data bytes
*/
static uint32_t USBH_PipeTransferGetResult (ARM_USBH_PIPE_HANDLE pipe_hndl) {

  if (pipe_hndl == 0U) { return 0U; }

  return (pipe[USBH_CH_GetIndexFromAddress((USB_HC_TypeDef *)pipe_hndl)].num_transferred_total);
}

/**
  \fn          int32_t USBH_PipeTransferAbort (ARM_USBH_PIPE_HANDLE pipe_hndl)
  \brief       Abort current USB Pipe transfer.
  \param[in]   pipe_hndl  Pipe Handle
  \return      \ref execution_status
*/
static int32_t USBH_PipeTransferAbort (ARM_USBH_PIPE_HANDLE pipe_hndl) {
  PIPE_t *ptr_pipe;

  if ((usb_drv_state & USBH_DRIVER_POWERED) == 0U) { return ARM_DRIVER_ERROR;           }
  if (pipe_hndl == 0U)                             { return ARM_DRIVER_ERROR_PARAMETER; }

  ptr_pipe = (PIPE_t *)(&pipe[USBH_CH_GetIndexFromAddress ((USB_HC_TypeDef *)pipe_hndl)]);

  if (ptr_pipe->active != 0U) {
    ptr_pipe->active = 0U;
    if (USBH_CH_Disable((USB_HC_TypeDef *)(pipe_hndl)) == 0U) { return ARM_DRIVER_ERROR; }
  }

  return ARM_DRIVER_OK;
}

/**
  \fn          uint16_t USBH_GetFrameNumber (void)
  \brief       Get current USB Frame Number.
  \return      Frame Number
*/
static uint16_t USBH_GetFrameNumber (void) {

  if ((usb_drv_state & USBH_DRIVER_POWERED) == 0U) { return 0U; }

  return ((USB->HFNUM >> 3) & 0x7FFU);
}

/**
  \fn          void USBH_IRQ (uint32_t gintsts)
  \brief       USB Host Interrupt Routine (IRQ).
*/
void USBH_IRQ (uint32_t gintsts) {
  PIPE_t            *ptr_pipe;
  USB_HC_TypeDef    *ptr_ch;
  uint8_t           *ptr_data;
  volatile uint32_t *dfifo;
  uint32_t           hprt, haint, hcint, pktcnt, xfrsiz, mpsiz, hcchar, hcchar_upd, transferred;
  uint32_t           grxsts, bcnt, ch, dat, len, len_rest;
  uint8_t            signal;

  if ((gintsts & USB_GINTSTS_PRTINT) != 0U) {           // If host port interrupt
    hprt = USB->HPRT;
    USB->HPRT = hprt & (~USB_HPRT_PRTENA);              // Leave PRTENA bit
    if ((hprt  & USB_HPRT_PRTCONNDET) != 0U) {          // Port connect detected
      if (port_reset == 0U) {                           // If port not under reset
        SignalPortEvent(0, ARM_USBH_EVENT_CONNECT);
      }
    }
    if ((hprt & USB_HPRT_PRTENCHNG) != 0U) {             // If port enable changed
      if ((hprt & USB_HPRT_PRTENA) != 0U) {              // If device connected
        if (port_reset != 0U) {
          port_reset = false;
          SignalPortEvent(0, ARM_USBH_EVENT_RESET);
        }
      }
    }
  }
  if ((gintsts & _USB_GINTSTS_DISCONNINT_MASK) != 0U) { // If device disconnected
    USB->GINTSTS = USB_GINTSTS_DISCONNINT;              // Clear disconnect interrupt
    if (port_reset == 0U) {                             // Ignore disconnect under reset*/
      ptr_ch   = (USB_HC_TypeDef *)(&USB->HC[0].CHAR);
      ptr_pipe = (PIPE_t         *)(pipe);
      for (ch = 0U; ch < USBH_MAX_PIPE_NUM; ch++) {
        if (ptr_pipe->active != 0U) {
          ptr_pipe->active = 0U;
          SignalPipeEvent((ARM_USBH_EP_HANDLE)ptr_ch, ARM_USBH_EVENT_BUS_ERROR);
        }
        ptr_ch++;
        ptr_pipe++;
      }
      SignalPortEvent(0, ARM_USBH_EVENT_DISCONNECT);
    }
  }
                                                        // Handle reception interrupt
  if ((gintsts & USB_GINTSTS_RXFLVL) != 0U) {           // If RXFIFO non-empty interrupt
    USB->GINTMSK &= ~USB_GINTSTS_RXFLVL;
    grxsts = USB->GRXSTSR;
    if (((grxsts >> 17) & 0x0FU) == 0x02U) {            // If PKTSTS = 0x02
      grxsts     = (USB->GRXSTSP);
      ch         = (grxsts     ) & 0x00FU;
      bcnt       = (grxsts >> 4) & 0x7FFU;
      dfifo      =  OTG_DFIFO[ch];
      ptr_data   =  pipe[ch].data + pipe[ch].num_transferred_total;
      len        =  bcnt / 4U;                          // Received number of 32-bit data
      len_rest   =  bcnt & 3U;                          // Number of bytes left
      while (len != 0U) {
        *((__packed uint32_t *)ptr_data) = *dfifo;
        ptr_data += 4U;
        len--;
      }
      if (len_rest != 0U) {
        dat = *((__packed uint32_t *)dfifo);
        while (len_rest != 0U) {
          *ptr_data++ = dat;
          dat >>= 8;
          len_rest--;
        }
      }
      pipe[ch].num_transferring      += bcnt;
      pipe[ch].num_transferred_total += bcnt;
    } else {                                            // If PKTSTS != 0x02
      grxsts      = USB->GRXSTSP;
    }
    USB->GINTMSK |= USB_GINTSTS_RXFLVL;
  }
                                                        // Handle host ctrl interrupt
  if ((gintsts & USB_GINTSTS_HCHINT) != 0U) {           // If host channel interrupt
    haint = USB->HAINT;
    for (ch = 0U; ch < USBH_MAX_PIPE_NUM; ch++) {
      if (haint == 0U) { break; }
      if ((haint & (1U << ch)) != 0U) {                 // If channels interrupt active
        haint     &= ~(1U << ch);
        signal     =   0U;
        ptr_ch     =  (USB_HC_TypeDef *)(&USB->HC[0].CHAR) + ch;
        ptr_pipe   =  (PIPE_t         *)(&pipe[ch]);
        hcint      =   ptr_ch->INT & ptr_ch->INTMSK;
        hcchar     =   ptr_ch->CHAR;
        hcchar_upd =   0U;
        if ((hcint & USB_HC_INT_XFERCOMPL) != 0U) {     // If data transfer finished
          if ((ptr_ch->CHAR & (1U << 15)) != 0U) {      // If endpoint IN
            ptr_pipe->event = ARM_USBH_EVENT_TRANSFER_COMPLETE;
          } else {                                      // If endpoint OUT
            ptr_pipe->num_transferred_total += pipe[ch].num_transferring;
            ptr_pipe->num_transferring       = 0U;
            if (ptr_pipe->num_transferred_total == pipe[ch].num) {
              ptr_pipe->event = ARM_USBH_EVENT_TRANSFER_COMPLETE;
            }
          }
          goto halt_ch;
        } else if (((hcint & USB_HC_INT_STALL     ) != 0U) ||   // If STALL received
                   ((hcint & USB_HC_INT_NAK       ) != 0U) ||   // If NAK received
                   ((hcint & USB_HC_INT_XACTERR   ) != 0U) ||   // If TXERR received
                   ((hcint & USB_HC_INT_BBLERR    ) != 0U) ||   // If BBERR received
                   ((hcint & USB_HC_INT_DATATGLERR) != 0U)) {   // If DTERR received
                                                        // Update transfer info
          if ((ptr_ch->CHAR & (1U << 15)) == 0U) {      // If endpoint OUT
            // Calculate number of successfully transferred bytes for Endpoint
            // OUT for Endpoint IN number of transferred bytes are handled on
            // RXFIFO non-empty interrupt
            pktcnt = (ptr_ch->TSIZ >> 19) & 0x3FFU;
            xfrsiz = (ptr_ch->TSIZ      ) & 0x7FFFFU;
            mpsiz  = (ptr_ch->CHAR      ) & 0x7FFU;
            if (ptr_pipe->num_transferring >= mpsiz) {
              if (pktcnt > 0U) {
                transferred = pipe[ch].num_transferring - ((mpsiz * (pktcnt-1)) + xfrsiz);
              } else {
                transferred = 0U;
              }
            } else {
              transferred = pipe[ch].num_transferring - xfrsiz;
            }
            ptr_pipe->num_transferred_total += transferred;
            ptr_pipe->num_transferring       = 0U;
          }

          if ((hcint & USB_HC_INT_STALL) != 0U) {       // If STALL received
            ptr_pipe->event = ARM_USBH_EVENT_HANDSHAKE_STALL;
            goto halt_ch;
          } else if ((hcint & USB_HC_INT_NAK) != 0U) {  // If NAK received
            // On NAK, NAK is not returned to middle layer but transfer is
            // restarted from driver for remaining data
            if ((ptr_ch->CHAR & (1U << 15)) != 0U) {    // If endpoint IN
              if (ptr_pipe->ep_type == ARM_USB_ENDPOINT_INTERRUPT) {
                goto halt_ch;
              } else {
                if ((USB->HPRT & USB_HPRT_PRTCONNSTS) != 0U) {
                  hcchar_upd = hcchar | USB_HC_CHAR_CHENA;
                } else {
                  goto halt_ch;
                }
              }
            } else {                                    // If endpoint OUT
              goto halt_ch;
            }
          } else {
            ptr_pipe->event = ARM_USBH_EVENT_BUS_ERROR;
            goto halt_ch;
          }
        } else if ((hcint & USB_HC_INTMSK_CHHLTDMSK) != 0U) {   // If channel halted
                                                        // Transfer is done here
          ptr_ch->INTMSK = 0U;                          // Mask all interrupts
          hcint = 0x7BBU;                               // Clear all interrupts
          ptr_pipe->in_progress = 0U;
          if (ptr_pipe->event != 0U) {
            ptr_pipe->active = 0U;
            signal = 1U;
          }
        } else if ((hcint & USB_HC_INTMSK_ACKMSK) != 0U) {      // If ACK received
          // On ACK, ACK is not an event that can be returned so when channel
          // is halted it will be signalled to middle layer if transfer is
          // completed otherwise transfer will be restarted for remaining
          // data
          if ((ptr_ch->CHAR & (1U << 15)) != 0U) {      // If endpoint IN
            if ((ptr_pipe->num != pipe[ch].num_transferred_total) &&                    // If all data was not transferred
                (ptr_pipe->num_transferring != 0U)                &&                    // If zero-length packet was not received
               ((ptr_pipe->num_transferred_total%ptr_pipe->ep_max_packet_size) == 0U)){ // If short packet was not received
              if ((USB->HPRT & USB_HPRT_PRTCONNSTS) != 0U) {
                hcchar_upd = hcchar | USB_HC_CHAR_CHENA;
              } else {
                goto halt_ch;
              }
            }
          }
        } else {
halt_ch:                                                // Halt the channel
          ptr_ch->INTMSK = USB_HC_INTMSK_CHHLTDMSK;
          hcchar_upd = hcchar | USB_HC_CHAR_CHENA | USB_HC_CHAR_CHDIS;
        }
        ptr_ch->INT = hcint;
        if (signal     != 0U) { SignalPipeEvent((ARM_USBH_EP_HANDLE)ptr_ch, ptr_pipe->event); }
        if (hcchar_upd != 0U) { ptr_ch->CHAR = hcchar_upd; }
      }
    }
  }

  // Handle periodic transfer timings
  if ((gintsts & USB_GINTMSK_SOFMSK) != 0U) {           // If start of frame interrupt
    USB->GINTSTS = USB_GINTSTS_SOF;                     // Clear SOF interrupt
    ptr_pipe     = (PIPE_t *)(pipe);
    for (ch = 0U; ch < USBH_MAX_PIPE_NUM; ch++) {
      // If interrupt transfer is active handle period (interval)
      if ((ptr_pipe->ep_type == ARM_USB_ENDPOINT_INTERRUPT) && (ptr_pipe->active != 0U) && (ptr_pipe->interval != 0U)) {
        ptr_pipe->interval--;
      }
      ptr_pipe++;
    }
  }

  // Handle restarts of unfinished transfers (due to NAK or ACK)
  ptr_pipe = (PIPE_t *)(pipe);
  for (ch = 0U; ch < USBH_MAX_PIPE_NUM; ch++) {
    if ((ptr_pipe->active != 0U) && (ptr_pipe->in_progress == 0U)) {
      // Restart periodic transfer if not in progress and interval expired
      if (ptr_pipe->ep_type == ARM_USB_ENDPOINT_INTERRUPT) {
        if (ptr_pipe->interval == 0U) {
          ptr_pipe->interval = ptr_pipe->interval_reload;
        } else {
          continue;
        }
      }

      ptr_pipe->in_progress = 1;
      if (USBH_HW_StartTransfer (ptr_pipe, USBH_CH_GetAddressFromIndex (ch)) == 0U) {
        ptr_pipe->in_progress = 0U;
        ptr_pipe->active      = 0U;
      }
    }
    ptr_pipe++;
  }
}

ARM_DRIVER_USBH Driver_USBH0 = {
  USBH_GetVersion,
  USBH_GetCapabilities,
  USBH_Initialize,
  USBH_Uninitialize,
  USBH_PowerControl,
  USBH_PortVbusOnOff,
  USBH_PortReset,
  USBH_PortSuspend,
  USBH_PortResume,
  USBH_PortGetState,
  USBH_PipeCreate,
  USBH_PipeModify,
  USBH_PipeDelete,
  USBH_PipeReset,
  USBH_PipeTransfer,
  USBH_PipeTransferGetResult,
  USBH_PipeTransferAbort,
  USBH_GetFrameNumber
};
